
###
### gRPC libraries and include files are provided with the casatools
### module to ensure compatibility
###
GRPC_LIBS = $$system(scripts/grpc-config --grpc-link)
GRPC_INCLUDE_raw = $$system(scripts/grpc-config --grpc-compile)
GRPC_INCLUDE = $$replace(GRPC_INCLUDE_raw,-I,)
GRPC_PATH = $$system(scripts/grpc-config --grpc-libpath)

system(scripts/do-config)

QMAKE_MOC = $$QMAKE_MOC -DAIPS_HAS_QWT
INCLUDEPATH += include include/casagrpc/protos build/include.$$QMAKE_HOST.version src/code casa-source/casatools/src/code casa-source/casatools/casacore sakura-source/libsakura/src

QMAKE_CXXFLAGS += -DAIPS_HAS_QWT -DUSE_GRPC -DUseCasacoreNamespace -DNO_CRASH_REPORTER -DAIPS_LINUX -DCASA6

unix:!mac{
  QMAKE_CXXFLAGS += -std=c++11
  CONFIG += release cpp11
  exists ( $$[QT_INSTALL_DATA]/mkspecs/features/qwt.prf ) {
    include ( $$[QT_INSTALL_DATA]/mkspecs/features/qwt.prf )
  }
  exists ( $$[QTMAKEFEATURES]/mkspecs/features/qwt.prf ) {
    include ( $$[QTMAKEFEATURES]/mkspecs/features/qwt.prf )
  }
  INCLUDEPATH += $$GRPC_INCLUDE
  INCLUDEPATH += /opt/casa/03/include /opt/casa/02/include /usr/include/cfitsio
  LIBDIR_LIST = /opt/casa/03/lib /opt/casa/02/lib /opt/rh/devtoolset-4/root/usr/lib/gcc/x86_64-redhat-linux/5.3.1 /opt/local/lib/gcc5
  GFORTRAN = gfortran
  ###
  ### -lelf is needed simply to cause it to be located from within the casaplotms.app
  ### tree at runtime by the runtime loader... because whatever library depends upon
  ### libelf.so.1 does not include a RPATH/RUNPATH
  ###
  LIBS += -lsqlite3 $$GRPC_LIBS -lwcs -lcfitsio -llapack -lblas -lgsl -lgslcblas -lfftw3 -lfftw3f -lfftw3_threads -lfftw3f_threads -lm -lgfortran -lelf -lrt -ldl
  greaterThan(QT_MAJOR_VERSION, 4) {
    LIBDIR_LIST += /usr/lib64
    LIBS += -lqwt-qt5
  }
  APP_EXE = build/bin.$$QMAKE_HOST.version/$${TARGET}-x86_64.AppImage
}

macx:{
  QMAKE_APPLE_DEVICE_ARCHS = $$system(uname -m)
  QMAKE_CXXFLAGS += -std=c++14 -arch $$QMAKE_APPLE_DEVICE_ARCHS -flat_namespace
  message ("OSX_FLAGS: $$OSX_FLAGS")
  GCC_VERSION="5"
  OSVERSION_MAJOR = $$system(uname -r | cut -d. -f1)
  greaterThan(OSVERSION_MAJOR, 19) {
    GCC_VERSION="9"
  }
  greaterThan(OSVERSION_MAJOR, 21) {
    GCC_VERSION="12"
  }
  message("GCC VERSION: $${GCC_VERSION}")
  DEFINES += NDEBUG
  CONFIG += c++14 
  MAC_THIRD_PARTY_PATH = "/opt/local"
  exists( /opt/casa/02 ) {
    MAC_THIRD_PARTY_PATH = "/opt/casa/02"
    message("Found /opt/casa/02")
  }
  exists( /opt/casa/03 ) {
    MAC_THIRD_PARTY_PATH = "/opt/casa/03"
    message("Found /opt/casa/03")
  }
  exists ( $$MAC_THIRD_PARTY_PATH/libexec/qt5/mkspecs/features/qwt.prf ) {
    include ( $$MAC_THIRD_PARTY_PATH/libexec/qt5/mkspecs/features/qwt.prf )
  }
  else {
    include ( $$MAC_THIRD_PARTY_PATH/libexec/qt4/share/mkspecs/features/qwt.prf )
  }

  LIBDIR_LIST = $$MAC_THIRD_PARTY_PATH/lib
  LIBS += -lwcs
  INCLUDEPATH += $$GRPC_INCLUDE
  INCLUDEPATH += $$MAC_THIRD_PARTY_PATH/include                #include <wcslib/wcs.h>
  exists ( $$MAC_THIRD_PARTY_PATH/libexec/qt5/mkspecs/features/qwt.prf ) {
  }
  else {
    INCLUDEPATH += $$MAC_THIRD_PARTY_PATH/libexec/qt4/include/qwt
  }
  LIBDIR_LIST = $$MAC_THIRD_PARTY_PATH/lib $$MAC_THIRD_PARTY_PATH/lib/gcc$${GCC_VERSION}
  LIBDIR_LIST = $$MAC_THIRD_PARTY_PATH/lib $$MAC_THIRD_PARTY_PATH/lib/gcc$${GCC_VERSION}
  GFORTRAN = gfortran-mp-$${GCC_VERSION}
  LIBS += -lsqlite3 $$GRPC_LIBS -lwcs -lcfitsio -llapack -lblas -lgsl -lfftw3 -lfftw3f -lfftw3_threads -lfftw3f_threads -lm -lgfortran
}

OBJECTS_DIR = build/lib.$$QMAKE_HOST.version/objs
MOC_DIR = build/lib.$$QMAKE_HOST.version/moc
RCC_DIR = build/lib.$$QMAKE_HOST.version/rcc
UI_DIR = build/include.$$QMAKE_HOST.version/ui

CONFIG += object_parallel_to_source

QT += xml widgets printsupport

for(p, LIBDIR_LIST) {
    exists($$p):LIBS += -L$$p
    exists($$p):RPATH_LIBS += $$p
}
YACC_DIR = include/yacc
INCLUDEPATH += $$YACC_DIR
QMAKE_CC = ccache $$QMAKE_CC
QMAKE_CXX = ccache $$QMAKE_CXX

###
### without this qmake will generate all of the MOC bindings for
### Qt classes that are #ifdefed out...
###
MOC_DIR = build/lib.$$QMAKE_HOST.version/moc
load(moc)
QMAKE_MOC += -DUSE_GRPC

unix:!mac{
    DESTDIR = build/bin.$$QMAKE_HOST.version/casaplotms.app/usr/bin
    APP = $$DESTDIR/../..
    # suppress the default RPATH if you wish
    QMAKE_LFLAGS_RPATH=
    # add your own with quoting gyrations to make sure $ORIGIN gets to the command line unexpanded
    QMAKE_LFLAGS += "-Wl,-rpath,\'\$$ORIGIN/../lib\'"
    QMAKE_POST_LINK = scripts/create-app --no-data name=plotms exe=$$DESTDIR/$$TARGET app=$$APP lib=$$DESTDIR/../lib tgt=$$lower($$QMAKE_HOST.os) plugins=$$[QT_INSTALL_PLUGINS] $$GRPC_PATH $$RPATH_LIBS
    APP_EXE = build/bin.$$QMAKE_HOST.version/$${TARGET}-x86_64.AppImage
}

macx:{
    # suppress the default RPATH if you wish
    QMAKE_LFLAGS_RPATH=
    # add your own with quoting gyrations to make sure $ORIGIN gets to the command line unexpanded
    QMAKE_LFLAGS += "-Wl,-rpath,\'@loader_path/../Frameworks\'"
    DESTDIR = build/bin.$$QMAKE_HOST.version
    APP = $$DESTDIR/$${TARGET}.app
    QMAKE_POST_LINK = scripts/create-app --no-data name=plotms exe=$$DESTDIR/$${TARGET}.app/Contents/MacOS/$$TARGET app=$$APP lib=$$DESTDIR/$${TARGET}.app/Contents/Frameworks tgt=$$lower($$QMAKE_HOST.os) plugins=$$[QT_INSTALL_PLUGINS] $$GRPC_PATH $$MAC_THIRD_PARTY_PATH/lib $$RPATH_LIBS
    ICON = casaplotms.icns
    APP_EXE = $$APP
}

fortran.name = Generate .o file for ${QMAKE_FILE_BASE}.f
fortran.input = FORTRAN_SOURCE  # variable containing our input files
fortran.output = ${OBJECTS_DIR}/${QMAKE_FILE_BASE}.o
fortran.commands = $$GFORTRAN -c ${QMAKE_FILE_NAME} -o ${QMAKE_FILE_OUT} -std=legacy
fortran.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += fortran
PRE_TARGETDEPS += compiler_fortran_make_all

yacc_ycc.name = Generate .YCC file for ${QMAKE_FILE_BASE}.yy
yacc_ycc.input = YACC_FILES  # variable containing our input files
yacc_ycc.variable_out = HEADERS
yacc_ycc.commands = bison -y -p ${QMAKE_FILE_BASE} -d -o $$YACC_DIR/${QMAKE_FILE_BASE}.ycc ${QMAKE_FILE_NAME}
yacc_ycc.output = $$YACC_DIR/${QMAKE_FILE_BASE}.ycc
yacc_ycc.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += yacc_ycc
PRE_TARGETDEPS += compiler_yacc_ycc_make_all

lex_lcc.name = Generate .LCC file for ${QMAKE_FILE_BASE}.ll
lex_lcc.input = LEX_FILES  # variable containing our input files
lex_lcc.variable_out = HEADERS
lex_lcc.commands = /usr/bin/flex -P${QMAKE_FILE_BASE} -o$$YACC_DIR/${QMAKE_FILE_BASE}.lcc ${QMAKE_FILE_NAME}
lex_lcc.output = $$YACC_DIR/${QMAKE_FILE_BASE}.lcc
lex_lcc.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += lex_lcc
PRE_TARGETDEPS += compiler_lex_lcc_make_all

PROTO_PLOTMS = src/code/plotms/protos/plotms.proto
PROTO_SHUTDOWN = $$system(scripts/grpc-config --proto-shutdown)
PROTO_PING = $$system(scripts/grpc-config --proto-ping)
PROTOS = $$PROTO_PLOTMS \
         $$system(scripts/grpc-config --proto-registrar) \
         $$PROTO_SHUTDOWN $$PROTO_PING

PROTOC = $$system(scripts/grpc-config --grpc-protocpp)
PROTOPY = $$system(scripts/grpc-config --grpc-protopy)
include(grpc_protobuf.pri)

FORTRAN_SOURCE = casa-source/casatools/src/code/synthesis/fortran/fgridft.f \
                 casa-source/casatools/src/code/synthesis/fortran/fmosaic.f \
                 casa-source/casatools/casacore/scimath_f/convolvegridder.f \
                 casa-source/casatools/src/code/synthesis/fortran/wprojgrid.f \
                 casa-source/casatools/casacore/scimath_f/fftpak.f \
                 casa-source/casatools/casacore/scimath_f/parametricsolver.f \
                 casa-source/casatools/src/code/synthesis/fortran/leakantso.f \
                 casa-source/casatools/src/code/synthesis/fortran/fpbwproj.f \
                 casa-source/casatools/casacore/scimath_f/grdsf.f \
                 casa-source/casatools/casacore/scimath_f/dfftpak.f

YACC_FILES = casa-source/casatools/casacore/tables/TaQL/RecordGram.yy \
  casa-source/casatools/casacore/casa/Json/JsonGram.yy \
  casa-source/casatools/casacore/tables/TaQL/TableGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSStateGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSTimeGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSSpwGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSObservationGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSFieldGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSFeedGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSCorrGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSScanGram.yy \
  casa-source/casatools/casacore/images/Images/ImageExprGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSArrayGram.yy \
  casa-source/casatools/src/code/synthesis/CalLibrary/CalLibraryGram.yy

LEX_FILES = casa-source/casatools/casacore/tables/TaQL/RecordGram.ll \
  casa-source/casatools/casacore/casa/Json/JsonGram.ll \
  casa-source/casatools/casacore/tables/TaQL/TableGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSStateGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSTimeGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSSpwGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSObservationGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSFieldGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSFeedGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSCorrGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSScanGram.ll \
  casa-source/casatools/casacore/images/Images/ImageExprGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSArrayGram.ll \
  casa-source/casatools/src/code/synthesis/CalLibrary/CalLibraryGram.ll

SOURCES = casa-source/casatools/casacore/casa/Arrays/ArrayBase.cc \
          casa-source/casatools/casacore/casa/Arrays/ArrayError.cc \
          casa-source/casatools/casacore/casa/Arrays/ArrayPartMath.cc \
          casa-source/casatools/casacore/casa/Arrays/ArrayPosIter.cc \
          casa-source/casatools/casacore/casa/Arrays/ArrayUtil2.cc \
          casa-source/casatools/casacore/casa/Arrays/Array2.cc \
          casa-source/casatools/casacore/casa/Arrays/Array2Math.cc \
          casa-source/casatools/casacore/casa/Arrays/Array_tmpl.cc \
          casa-source/casatools/casacore/casa/Arrays/AxesMapping.cc \
          casa-source/casatools/casacore/casa/Arrays/AxesSpecifier.cc \
          casa-source/casatools/casacore/casa/Arrays/ExtendSpecifier.cc \
          casa-source/casatools/casacore/casa/Arrays/IPosition.cc \
          casa-source/casatools/casacore/casa/Arrays/Matrix2Math.cc \
          casa-source/casatools/casacore/casa/Arrays/Matrix_tmpl.cc \
          casa-source/casatools/casacore/casa/Arrays/Slice.cc \
          casa-source/casatools/casacore/casa/Arrays/Slicer.cc \
          casa-source/casatools/casacore/casa/Arrays/Vector_tmpl.cc \
          casa-source/casatools/casacore/casa/BasicMath/Math.cc \
          casa-source/casatools/casacore/casa/BasicMath/Random.cc \
          casa-source/casatools/casacore/casa/BasicSL/Complex.cc \
          casa-source/casatools/casacore/casa/BasicSL/Constants.cc \
          casa-source/casatools/casacore/casa/BasicSL/IComplex.cc \
          casa-source/casatools/casacore/casa/Utilities/Regex.cc \
          casa-source/casatools/casacore/casa/BasicSL/String.cc \
          casa-source/casatools/casacore/casa/Containers/Allocator.cc \
          casa-source/casatools/casacore/casa/Containers/Block.cc \
          casa-source/casatools/casacore/casa/Containers/Block_tmpl.cc \
          casa-source/casatools/casacore/casa/Containers/IterError.cc \
          casa-source/casatools/casacore/casa/Containers/Record.cc \
          casa-source/casatools/casacore/casa/Containers/Record2.cc \
          casa-source/casatools/casacore/casa/Containers/Record2Interface.cc \
          casa-source/casatools/casacore/casa/Containers/RecordDesc.cc \
          casa-source/casatools/casacore/casa/Containers/RecordDescRep.cc \
          casa-source/casatools/casacore/casa/Containers/RecordInterface.cc \
          casa-source/casatools/casacore/casa/Containers/RecordRep.cc \
          casa-source/casatools/casacore/casa/Containers/ValueHolder.cc \
          casa-source/casatools/casacore/casa/Containers/ValueHolderRep.cc \
          casa-source/casatools/casacore/casa/Exceptions/Error2.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5DataSet.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5DataType.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5Error.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5File.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5Group.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5HidMeta.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5Object.cc \
          casa-source/casatools/casacore/casa/HDF5/HDF5Record.cc \
          casa-source/casatools/casacore/casa/IO/AipsIO.cc \
          casa-source/casatools/casacore/casa/IO/BucketBase.cc \
          casa-source/casatools/casacore/casa/IO/BucketBuffered.cc \
          casa-source/casatools/casacore/casa/IO/BucketCache.cc \
          casa-source/casatools/casacore/casa/IO/BucketFile.cc \
          casa-source/casatools/casacore/casa/IO/BucketMapped.cc \
          casa-source/casatools/casacore/casa/IO/ByteIO.cc \
          casa-source/casatools/casacore/casa/IO/CanonicalIO.cc \
          casa-source/casatools/casacore/casa/IO/FileLocker.cc \
          casa-source/casatools/casacore/casa/IO/FilebufIO.cc \
          casa-source/casatools/casacore/casa/IO/FiledesIO.cc \
          casa-source/casatools/casacore/casa/IO/FileUnbufferedIO.cc \
          casa-source/casatools/casacore/casa/IO/IPositionIO.cc \
          casa-source/casatools/casacore/casa/IO/LECanonicalIO.cc \
          casa-source/casatools/casacore/casa/IO/LockFile.cc \
          casa-source/casatools/casacore/casa/IO/MFFileIO.cc \
          casa-source/casatools/casacore/casa/IO/MMapfdIO.cc \
          casa-source/casatools/casacore/casa/IO/MemoryIO.cc \
          casa-source/casatools/casacore/casa/IO/MultiFile.cc \
          casa-source/casatools/casacore/casa/IO/MultiFileBase.cc \
          casa-source/casatools/casacore/casa/IO/MultiHDF5.cc \
          casa-source/casatools/casacore/casa/IO/RegularFileIO.cc \
          casa-source/casatools/casacore/casa/IO/TypeIO.cc \
          casa-source/casatools/casacore/casa/Json/JsonError.cc \
          casa-source/casatools/casacore/casa/Json/JsonKVMap.cc \
          casa-source/casatools/casacore/casa/Json/JsonOut.cc \
          casa-source/casatools/casacore/casa/Json/JsonParser.cc \
          casa-source/casatools/casacore/casa/Json/JsonValue.cc \
          casa-source/casatools/casacore/casa/Logging/LogFilter.cc \
          casa-source/casatools/casacore/casa/Logging/LogFilterInterface.cc \
          casa-source/casatools/casacore/casa/Logging/LogIO.cc \
          casa-source/casatools/casacore/casa/Logging/LogMessage.cc \
          casa-source/casatools/casacore/casa/Logging/LogOrigin.cc \
          casa-source/casatools/casacore/casa/Logging/LogSink.cc \
          casa-source/casatools/casacore/casa/Logging/LogSinkInterface.cc \
          casa-source/casatools/casacore/casa/Logging/MemoryLogSink.cc \
          casa-source/casatools/casacore/casa/Logging/NullLogSink.cc \
          casa-source/casatools/casacore/casa/Logging/StreamLogSink.cc \
          casa-source/casatools/casacore/casa/OS/CanonicalConversion.cc \
          casa-source/casatools/casacore/casa/OS/Conversion.cc \
          casa-source/casatools/casacore/casa/OS/DOos.cc \
          casa-source/casatools/casacore/casa/OS/Directory.cc \
          casa-source/casatools/casacore/casa/OS/DirectoryIterator.cc \
          casa-source/casatools/casacore/casa/OS/DynLib.cc \
          casa-source/casatools/casacore/casa/OS/EnvVar.cc \
          casa-source/casatools/casacore/casa/OS/File.cc \
          casa-source/casatools/casacore/casa/OS/HostInfo.cc \
          casa-source/casatools/casacore/casa/OS/LECanonicalConversion.cc \
          casa-source/casatools/casacore/casa/OS/Memory.cc \
          casa-source/casatools/casacore/casa/OS/MemoryTrace.cc \
          casa-source/casatools/casacore/casa/OS/OMP.cc \
          casa-source/casatools/casacore/casa/OS/Path.cc \
          casa-source/casatools/casacore/casa/OS/RegularFile.cc \
          casa-source/casatools/casacore/casa/OS/SymLink.cc \
          casa-source/casatools/casacore/casa/OS/Time.cc \
          casa-source/casatools/casacore/casa/OS/Timer.cc \
          casa-source/casatools/casacore/casa/Quanta/Euler.cc \
          casa-source/casatools/casacore/casa/Quanta/MVAngle.cc \
          casa-source/casatools/casacore/casa/Quanta/MVBaseline.cc \
          casa-source/casatools/casacore/casa/Quanta/MVDirection.cc \
          casa-source/casatools/casacore/casa/Quanta/MVDoppler.cc \
          casa-source/casatools/casacore/casa/Quanta/MVEarthMagnetic.cc \
          casa-source/casatools/casacore/casa/Quanta/MVEpoch.cc \
          casa-source/casatools/casacore/casa/Quanta/MVFrequency.cc \
          casa-source/casatools/casacore/casa/Quanta/MVPosition.cc \
          casa-source/casatools/casacore/casa/Quanta/MVRadialVelocity.cc \
          casa-source/casatools/casacore/casa/Quanta/MVTime.cc \
          casa-source/casatools/casacore/casa/Quanta/MVuvw.cc \
          casa-source/casatools/casacore/casa/Quanta/MeasValue.cc \
          casa-source/casatools/casacore/casa/Quanta/QBase.cc \
          casa-source/casatools/casacore/casa/Quanta/QLogical2.cc \
          casa-source/casatools/casacore/casa/Quanta/QMath2.cc \
          casa-source/casatools/casacore/casa/Quanta/Quantum2.cc \
          casa-source/casatools/casacore/casa/Quanta/QuantumHolder.cc \
          casa-source/casatools/casacore/casa/Quanta/RotMatrix.cc \
          casa-source/casatools/casacore/casa/Quanta/Unit.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitDim.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap2.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap3.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap4.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap5.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap6.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitMap7.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitName.cc \
          casa-source/casatools/casacore/casa/Quanta/UnitVal.cc \
          casa-source/casatools/casacore/casa/System/Aipsrc.cc \
          casa-source/casatools/casacore/casa/System/AipsrcBool.cc \
          casa-source/casatools/casacore/casa/System/AipsrcVString.cc \
          casa-source/casatools/casacore/casa/System/AipsrcValue2.cc \
          casa-source/casatools/casacore/casa/System/AppInfo.cc \
          casa-source/casatools/casacore/casa/System/AppState.cc \
          casa-source/casatools/casacore/casa/System/Casarc.cc \
          casa-source/casatools/casacore/casa/System/Choice.cc \
          casa-source/casatools/casacore/casa/System/ObjectID.cc \
          casa-source/casatools/casacore/casa/System/PGPlotter.cc \
          casa-source/casatools/casacore/casa/System/PGPlotterInterface.cc \
          casa-source/casatools/casacore/casa/System/PGPlotterNull.cc \
          casa-source/casatools/casacore/casa/System/ProgressMeter.cc \
          casa-source/casatools/casacore/casa/Utilities/Compare.cc \
          casa-source/casatools/casacore/casa/Utilities/CompositeNumber.cc \
          casa-source/casatools/casacore/casa/Utilities/Copy2.cc \
          casa-source/casatools/casacore/casa/Utilities/CountedPtr2.cc \
          casa-source/casatools/casacore/casa/Utilities/DataType.cc \
          casa-source/casatools/casacore/casa/Utilities/MUString.cc \
          casa-source/casatools/casacore/casa/Utilities/Precision.cc \
          casa-source/casatools/casacore/casa/Utilities/RecordTransformable.cc \
          casa-source/casatools/casacore/casa/Utilities/Sort.cc \
          casa-source/casatools/casacore/casa/Utilities/SortError.cc \
          casa-source/casatools/casacore/casa/Utilities/StringDistance.cc \
          casa-source/casatools/casacore/casa/Utilities/ValType.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/Coordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/CoordinateSystem.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/CoordinateUtil.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/Direction2Coordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/DirectionCoordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/FITSCoordinateUtil.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/LinearCoordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/LinearXform.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/LinearXform2.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/ObsInfo.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/Projection.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/QualityCoordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/Spectral2Coordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/SpectralCoordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/StokesCoordinate.cc \
          casa-source/casatools/casacore/coordinates/Coordinates/TabularCoordinate.cc \
          casa-source/casatools/casacore/fits/FITS/BinTable.cc \
          casa-source/casatools/casacore/fits/FITS/FITSDateUtil.cc \
          casa-source/casatools/casacore/fits/FITS/FITSError.cc \
          casa-source/casatools/casacore/fits/FITS/FITSHistoryUtil.cc \
          casa-source/casatools/casacore/fits/FITS/FITSKeywordUtil.cc \
          casa-source/casatools/casacore/fits/FITS/FITSSpectralUtil.cc \
          casa-source/casatools/casacore/fits/FITS/FITSTable2.cc \
          casa-source/casatools/casacore/fits/FITS/blockio.cc \
          casa-source/casatools/casacore/fits/FITS/fits.cc \
          casa-source/casatools/casacore/fits/FITS/fitsio.cc \
          casa-source/casatools/casacore/fits/FITS/hdu2.cc \
          casa-source/casatools/casacore/images/Images/FITSErrorImage.cc \
          casa-source/casatools/casacore/images/Images/FITSImage.cc \
          casa-source/casatools/casacore/images/Images/FITSImgParser.cc \
          casa-source/casatools/casacore/images/Images/FITSQualityImage.cc \
          casa-source/casatools/casacore/images/Images/FITSQualityMask.cc \
          casa-source/casatools/casacore/images/Images/HDF5Image2.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrGroup.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrGroupCasa.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrGroupHDF5.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrHandler.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrHandlerCasa.cc \
          casa-source/casatools/casacore/images/Images/ImageAttrHandlerHDF5.cc \
          casa-source/casatools/casacore/images/Images/ImageBeamSet.cc \
          casa-source/casatools/casacore/images/Images/ImageExprGram.cc \
          casa-source/casatools/casacore/images/Images/ImageExprParse.cc \
          casa-source/casatools/casacore/images/Images/ImageFITS2Converter.cc \
          casa-source/casatools/casacore/images/Images/ImageInfo.cc \
          casa-source/casatools/casacore/images/Images/ImageOpener.cc \
          casa-source/casatools/casacore/images/Images/ImageUtilities.cc \
          casa-source/casatools/casacore/images/Images/Image_tmpl.cc \
          casa-source/casatools/casacore/images/Images/LELImageCoord.cc \
          casa-source/casatools/casacore/images/Images/MIRIADImage.cc \
          casa-source/casatools/casacore/images/Images/PagedImage2.cc \
          casa-source/casatools/casacore/images/Regions/ImageRegion.cc \
          casa-source/casatools/casacore/images/Regions/RegionHandler.cc \
          casa-source/casatools/casacore/images/Regions/RegionHandlerHDF5.cc \
          casa-source/casatools/casacore/images/Regions/RegionHandlerMemory.cc \
          casa-source/casatools/casacore/images/Regions/RegionHandlerTable.cc \
          casa-source/casatools/casacore/images/Regions/WCBox.cc \
          casa-source/casatools/casacore/images/Regions/WCComplement.cc \
          casa-source/casatools/casacore/images/Regions/WCCompound.cc \
          casa-source/casatools/casacore/images/Regions/WCConcatenation.cc \
          casa-source/casatools/casacore/images/Regions/WCDifference.cc \
          casa-source/casatools/casacore/images/Regions/WCEllipsoid.cc \
          casa-source/casatools/casacore/images/Regions/WCExtension.cc \
          casa-source/casatools/casacore/images/Regions/WCIntersection.cc \
          casa-source/casatools/casacore/images/Regions/WCLELMask.cc \
          casa-source/casatools/casacore/images/Regions/WCPolygon.cc \
          casa-source/casatools/casacore/images/Regions/WCRegion.cc \
          casa-source/casatools/casacore/images/Regions/WCRegion2.cc \
          casa-source/casatools/casacore/images/Regions/WCUnion.cc \
          casa-source/casatools/casacore/lattices/LEL/LELArrayBase.cc \
          casa-source/casatools/casacore/lattices/LEL/LELAttribute.cc \
          casa-source/casatools/casacore/lattices/LEL/LELBinary2.cc \
          casa-source/casatools/casacore/lattices/LEL/LELCoordinates.cc \
          casa-source/casatools/casacore/lattices/LEL/LELFunction2.cc \
          casa-source/casatools/casacore/lattices/LEL/LELLattCoord.cc \
          casa-source/casatools/casacore/lattices/LEL/LELLattCoordBase.cc \
          casa-source/casatools/casacore/lattices/LEL/LELRegion.cc \
          casa-source/casatools/casacore/lattices/LEL/LELUnary2.cc \
          casa-source/casatools/casacore/lattices/LEL/LatticeExprNode.cc \
          casa-source/casatools/casacore/lattices/LRegions/FITSMask.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCBox.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCComplement.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCConcatenation.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCDifference.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCEllipsoid.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCExtension.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCHDF5Mask.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCIntersection.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCLELMask.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCMask.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCPagedMask.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCPixelSet.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCPolygon.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCRegion.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCRegion2.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCRegionFixed.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCRegionMulti.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCRegionSingle.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCSlicer.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCStretch.cc \
          casa-source/casatools/casacore/lattices/LRegions/LCUnion.cc \
          casa-source/casatools/casacore/lattices/LRegions/LattRegionHolder.cc \
          casa-source/casatools/casacore/lattices/LRegions/LatticeRegion.cc \
          casa-source/casatools/casacore/lattices/LRegions/RegionType.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/Fit2D.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LattStatsProgress.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LattStatsSpecialize.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LatticeHistProgress.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LatticeHistSpecialize.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LatticeProgress.cc \
          casa-source/casatools/casacore/lattices/LatticeMath/LatticeStatsBase.cc \
          casa-source/casatools/casacore/lattices/Lattices/LatticeBase.cc \
          casa-source/casatools/casacore/lattices/Lattices/LatticeIndexer.cc \
          casa-source/casatools/casacore/lattices/Lattices/LatticeLocker.cc \
          casa-source/casatools/casacore/lattices/Lattices/LatticeNavigator.cc \
          casa-source/casatools/casacore/lattices/Lattices/LatticeStepper.cc \
          casa-source/casatools/casacore/lattices/Lattices/Lattices_tmpl.cc \
          casa-source/casatools/casacore/lattices/Lattices/PixelCurve1D.cc \
          casa-source/casatools/casacore/lattices/Lattices/TileStepper.cc \
          casa-source/casatools/casacore/lattices/Lattices/TiledLineStepper.cc \
          casa-source/casatools/casacore/lattices/Lattices/TiledShape.cc \
          casa-source/casatools/casacore/measures/Measures/Aberration.cc \
          casa-source/casatools/casacore/measures/Measures/EarthField.cc \
          casa-source/casatools/casacore/measures/Measures/EarthMagneticMachine.cc \
          casa-source/casatools/casacore/measures/Measures/MBaseline.cc \
          casa-source/casatools/casacore/measures/Measures/MCBase.cc \
          casa-source/casatools/casacore/measures/Measures/MCBaseline.cc \
          casa-source/casatools/casacore/measures/Measures/MCDirection.cc \
          casa-source/casatools/casacore/measures/Measures/MCDoppler.cc \
          casa-source/casatools/casacore/measures/Measures/MCEpoch.cc \
          casa-source/casatools/casacore/measures/Measures/MCFrame.cc \
          casa-source/casatools/casacore/measures/Measures/MCFrequency.cc \
          casa-source/casatools/casacore/measures/Measures/MCPosition.cc \
          casa-source/casatools/casacore/measures/Measures/MCRadialVelocity.cc \
          casa-source/casatools/casacore/measures/Measures/MConvertBase.cc \
          casa-source/casatools/casacore/measures/Measures/MDirection.cc \
          casa-source/casatools/casacore/measures/Measures/MDoppler.cc \
          casa-source/casatools/casacore/measures/Measures/MEarthMagnetic.cc \
          casa-source/casatools/casacore/measures/Measures/MEpoch.cc \
          casa-source/casatools/casacore/measures/Measures/MFrequency.cc \
          casa-source/casatools/casacore/measures/Measures/MPosition.cc \
          casa-source/casatools/casacore/measures/Measures/MRBase.cc \
          casa-source/casatools/casacore/measures/Measures/MRadialVelocity.cc \
          casa-source/casatools/casacore/measures/Measures/MeasComet.cc \
          casa-source/casatools/casacore/measures/Measures/MeasData.cc \
          casa-source/casatools/casacore/measures/Measures/MeasFrame.cc \
          casa-source/casatools/casacore/measures/Measures/MeasIERS.cc \
          casa-source/casatools/casacore/measures/Measures/MeasJPL.cc \
          casa-source/casatools/casacore/measures/Measures/MeasMath.cc \
          casa-source/casatools/casacore/measures/Measures/MeasTable.cc \
          casa-source/casatools/casacore/measures/Measures/MeasTableMul.cc \
          casa-source/casatools/casacore/measures/Measures/Measure.cc \
          casa-source/casatools/casacore/measures/Measures/MeasureHolder.cc \
          casa-source/casatools/casacore/measures/Measures/Muvw.cc \
          casa-source/casatools/casacore/measures/Measures/Nutation.cc \
          casa-source/casatools/casacore/measures/Measures/Precession.cc \
          casa-source/casatools/casacore/measures/Measures/Quality.cc \
          casa-source/casatools/casacore/measures/Measures/SolarPos.cc \
          casa-source/casatools/casacore/measures/Measures/Stokes.cc \
          casa-source/casatools/casacore/measures/Measures/UVWMachine.cc \
          casa-source/casatools/casacore/measures/Measures/VelocityMachine.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasColumn.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasDescBase.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasOffsetDesc.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasRefDesc.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasType.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableMeasValueDesc.cc \
          casa-source/casatools/casacore/measures/TableMeasures/TableQuantumDesc.cc \
          casa-source/casatools/casacore/ms/MSOper/MSDerivedValues.cc \
          casa-source/casatools/casacore/ms/MSOper/MSKeys.cc \
          casa-source/casatools/casacore/ms/MSOper/MSMetaData.cc \
          casa-source/casatools/casacore/ms/MSOper/MSSummary.cc \
          casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSAntennaIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSAntennaParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSArrayGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSArrayParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSCorrGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSCorrParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSDataDescIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFeedGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFeedIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFeedParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFieldGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFieldIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSFieldParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSObservationGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSObservationParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSPolIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSPolnGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSPolnParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSSpwErrorHandler.cc \
          casa-source/casatools/casacore/ms/MSSel/MSScanGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSScanParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelectableTable.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelection.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelectionError.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelectionErrorHandler.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelectionKeywords.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelectionTools.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSelector.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSourceIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSpWindowIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSpwGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSpwIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSSpwParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSStateGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSStateIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSStateParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSTableIndex.cc \
          casa-source/casatools/casacore/ms/MSSel/MSTimeGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSTimeParse.cc \
          casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.cc \
          casa-source/casatools/casacore/ms/MSSel/MSUvDistParse.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSAntenna.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSAntennaColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSDataDescColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSDataDescription.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSDoppler.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSDopplerColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSDopplerUtil.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFeed.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFeedColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSField.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFieldColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFlagCmd.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFlagCmdColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFreqOffColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSFreqOffset.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSHistory.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSHistoryColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSHistoryHandler.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSIter.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSMainColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSObsColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSObservation.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSPointing.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSPointingColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSPolColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSPolarization.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSProcessor.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSProcessorColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSRange.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSource.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSourceColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSpWindowColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSpectralWindow.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSState.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSStateColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSysCal.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSSysCalColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSTable.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSTableImpl.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSTileLayout.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSWeather.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MSWeatherColumns.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/MeasurementSet.cc \
          casa-source/casatools/casacore/ms/MeasurementSets/StokesConverter.cc \
          casa-source/casatools/casacore/scimath/Fitting/LSQFit.cc \
          casa-source/casatools/casacore/scimath/Fitting/LSQMatrix.cc \
          casa-source/casatools/casacore/scimath/Fitting/LSQMatrix2.cc \
          casa-source/casatools/casacore/scimath/Functionals/FuncExprData.cc \
          casa-source/casatools/casacore/scimath/Functionals/FuncExpression.cc \
          casa-source/casatools/casacore/scimath/Mathematics/Combinatorics.cc \
          casa-source/casatools/casacore/scimath/Mathematics/FFTPack.cc \
          casa-source/casatools/casacore/scimath/Mathematics/FFTServer.cc \
          casa-source/casatools/casacore/scimath/Mathematics/FFTW.cc \
          casa-source/casatools/casacore/scimath/Mathematics/GaussianBeam.cc \
          casa-source/casatools/casacore/scimath/Mathematics/Interpolate2D.cc \
          casa-source/casatools/casacore/scimath/Mathematics/MathFunc2.cc \
          casa-source/casatools/casacore/scimath/Mathematics/NumericTraits.cc \
          casa-source/casatools/casacore/scimath/Mathematics/SquareMatrix2.cc \
          casa-source/casatools/casacore/scimath/StatsFramework/BiweightStatisticsData.cc \
          casa-source/casatools/casacore/scimath/StatsFramework/ClassicalStatisticsData.cc \
          casa-source/casatools/casacore/scimath/StatsFramework/StatisticsData.cc \
          casa-source/casatools/casacore/scimath/StatsFramework/ZScoreCalculator.cc \
          casa-source/casatools/casacore/tables/DataMan/BitFlagsEngine.cc \
          casa-source/casatools/casacore/tables/DataMan/DataManagerColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/VirtArrCol.cc \
          casa-source/casatools/casacore/tables/DataMan/CompressComplex.cc \
          casa-source/casatools/casacore/tables/DataMan/CompressFloat.cc \
          casa-source/casatools/casacore/tables/DataMan/DataManAccessor.cc \
          casa-source/casatools/casacore/tables/DataMan/DataManError.cc \
          casa-source/casatools/casacore/tables/DataMan/DataManInfo.cc \
          casa-source/casatools/casacore/tables/DataMan/DataManager.cc \
          casa-source/casatools/casacore/tables/DataMan/ForwardCol.cc \
          casa-source/casatools/casacore/tables/DataMan/ISMBase.cc \
          casa-source/casatools/casacore/tables/DataMan/ISMBucket.cc \
          casa-source/casatools/casacore/tables/DataMan/ISMColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/ISMIndColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/ISMIndex.cc \
          casa-source/casatools/casacore/tables/DataMan/IncrStManAccessor.cc \
          casa-source/casatools/casacore/tables/DataMan/IncrementalStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/MSMBase.cc \
          casa-source/casatools/casacore/tables/DataMan/MSMColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/MSMDirColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/MSMIndColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/MemoryStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMBase.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMDirColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMIndColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMIndStringColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMIndex.cc \
          casa-source/casatools/casacore/tables/DataMan/SSMStringHandler.cc \
          casa-source/casatools/casacore/tables/DataMan/StArrAipsIO.cc \
          casa-source/casatools/casacore/tables/DataMan/StArrayFile.cc \
          casa-source/casatools/casacore/tables/DataMan/StIndArrAIO.cc \
          casa-source/casatools/casacore/tables/DataMan/StIndArray.cc \
          casa-source/casatools/casacore/tables/DataMan/StManAipsIO.cc \
          casa-source/casatools/casacore/tables/DataMan/StManColumnBase.cc \
          casa-source/casatools/casacore/tables/DataMan/StManColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/StandardStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/StandardStManAccessor.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMCoordColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMCube.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMCubeBuff.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMCubeMMap.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMDataColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMFile.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMIdColumn.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMOption.cc \
          casa-source/casatools/casacore/tables/DataMan/TSMShape.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledCellStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledColumnStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledDataStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledFileAccess.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledFileHelper.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledShapeStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledStMan.cc \
          casa-source/casatools/casacore/tables/DataMan/TiledStManAccessor.cc \
          casa-source/casatools/casacore/tables/DataMan/VirtColEng.cc \
          casa-source/casatools/casacore/tables/DataMan/VirtualTaQLColumn.cc \
          casa-source/casatools/casacore/tables/LogTables/LoggerHolder.cc \
          casa-source/casatools/casacore/tables/LogTables/NewFile.cc \
          casa-source/casatools/casacore/tables/LogTables/TableLogSink.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprAggrNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprAggrNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprConeNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprDerNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprDerNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprFuncNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprFuncNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprGroup.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprGroupAggrFunc.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprGroupAggrFuncArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprLogicNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprLogicNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprMathNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprMathNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeRecord.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeRep.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeSet.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeSetElem.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeSetOpt.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprNodeUtil.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprRange.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprUDFNode.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprUDFNodeArray.cc \
          casa-source/casatools/casacore/tables/TaQL/ExprUnitNode.cc \
          casa-source/casatools/casacore/tables/TaQL/MArrayBase.cc \
          casa-source/casatools/casacore/tables/TaQL/RecordExpr.cc \
          casa-source/casatools/casacore/tables/TaQL/RecordGram.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLJoin.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLNode.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLNodeDer.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLNodeHandler.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLNodeRep.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLNodeVisitor.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLResult.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLShow.cc \
          casa-source/casatools/casacore/tables/TaQL/TaQLStyle.cc \
          casa-source/casatools/casacore/tables/TaQL/TableGram.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParse.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseJoin.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseFunc.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseGroupby.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseProject.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseQuery.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseSortKey.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseTableList.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseUpdate.cc \
          casa-source/casatools/casacore/tables/TaQL/TableParseUtil.cc \
          casa-source/casatools/casacore/tables/TaQL/UDFBase.cc \
          casa-source/casatools/casacore/tables/Tables/ArrColData.cc \
          casa-source/casatools/casacore/tables/Tables/ArrColDesc.cc \
          casa-source/casatools/casacore/tables/Tables/ArrColDesc_tmpl.cc \
          casa-source/casatools/casacore/tables/Tables/ArrayColumnBase.cc \
          casa-source/casatools/casacore/tables/Tables/ArrayColumn_tmpl.cc \
          casa-source/casatools/casacore/tables/Tables/BaseColDesc.cc \
          casa-source/casatools/casacore/tables/Tables/BaseColumn.cc \
          casa-source/casatools/casacore/tables/Tables/BaseTabIter.cc \
          casa-source/casatools/casacore/tables/Tables/BaseTable.cc \
          casa-source/casatools/casacore/tables/Tables/ColDescSet.cc \
          casa-source/casatools/casacore/tables/Tables/ColumnCache.cc \
          casa-source/casatools/casacore/tables/Tables/ColumnDesc.cc \
          casa-source/casatools/casacore/tables/Tables/ColumnSet.cc \
          casa-source/casatools/casacore/tables/Tables/ColumnsIndex.cc \
          casa-source/casatools/casacore/tables/Tables/ConcatColumn.cc \
          casa-source/casatools/casacore/tables/Tables/ConcatRows.cc \
          casa-source/casatools/casacore/tables/Tables/ConcatTable.cc \
          casa-source/casatools/casacore/tables/Tables/MemoryTable.cc \
          casa-source/casatools/casacore/tables/Tables/NullTable.cc \
          casa-source/casatools/casacore/tables/Tables/PlainColumn.cc \
          casa-source/casatools/casacore/tables/Tables/PlainTable.cc \
          casa-source/casatools/casacore/tables/Tables/ReadAsciiTable.cc \
          casa-source/casatools/casacore/tables/Tables/RefColumn.cc \
          casa-source/casatools/casacore/tables/Tables/RefRows.cc \
          casa-source/casatools/casacore/tables/Tables/RefTable.cc \
          casa-source/casatools/casacore/tables/Tables/RowCopier.cc \
          casa-source/casatools/casacore/tables/Tables/RowNumbers.cc \
          casa-source/casatools/casacore/tables/Tables/ScaColDesc_tmpl.cc \
          casa-source/casatools/casacore/tables/Tables/ScaRecordColData.cc \
          casa-source/casatools/casacore/tables/Tables/ScaRecordColDesc.cc \
          casa-source/casatools/casacore/tables/Tables/ScalarColumn_tmpl.cc \
          casa-source/casatools/casacore/tables/Tables/SetupNewTab.cc \
          casa-source/casatools/casacore/tables/Tables/StorageOption.cc \
          casa-source/casatools/casacore/tables/Tables/SubTabDesc.cc \
          casa-source/casatools/casacore/tables/Tables/TabPath.cc \
          casa-source/casatools/casacore/tables/Tables/Table.cc \
          casa-source/casatools/casacore/tables/Tables/TableAttr.cc \
          casa-source/casatools/casacore/tables/Tables/TableCache.cc \
          casa-source/casatools/casacore/tables/Tables/TableColumn.cc \
          casa-source/casatools/casacore/tables/Tables/TableCopy.cc \
          casa-source/casatools/casacore/tables/Tables/TableDesc.cc \
          casa-source/casatools/casacore/tables/Tables/TableError.cc \
          casa-source/casatools/casacore/tables/Tables/TableInfo.cc \
          casa-source/casatools/casacore/tables/Tables/TableIter.cc \
          casa-source/casatools/casacore/tables/Tables/TableIterProxy.cc \
          casa-source/casatools/casacore/tables/Tables/TableKeyword.cc \
          casa-source/casatools/casacore/tables/Tables/TableLock.cc \
          casa-source/casatools/casacore/tables/Tables/TableLockData.cc \
          casa-source/casatools/casacore/tables/Tables/TableLocker.cc \
          casa-source/casatools/casacore/tables/Tables/TableProxy.cc \
          casa-source/casatools/casacore/tables/Tables/TableRecord.cc \
          casa-source/casatools/casacore/tables/Tables/TableRecordRep.cc \
          casa-source/casatools/casacore/tables/Tables/TableRow.cc \
          casa-source/casatools/casacore/tables/Tables/TableSyncData.cc \
          casa-source/casatools/casacore/tables/Tables/TableTrace.cc \
          casa-source/casatools/casacore/tables/Tables/TableUtil.cc \
          casa-source/casatools/src/code/asdmstman/AsdmColumn.cc \
          casa-source/casatools/src/code/asdmstman/AsdmIndex.cc \
          casa-source/casatools/src/code/asdmstman/AsdmStMan.cc \
          casa-source/casatools/src/code/asdmstman/Register.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMAngle.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMException.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMFrequency.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMInverseLength.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMLength.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMMassDensity.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMNumberDensity.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMOpacity.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMPercent.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMPressure.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMProfile.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMRefractiveIndex.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMRefractiveIndexProfile.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMSkyStatus.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMSpectralGrid.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMTemperature.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMWVRMeasurement.cc \
          casa-source/casatools/src/code/atmosphere/ATM/ATMWaterVaporRadiometer.cc \
          src/code/casaqt/PlotterImplementations/PlotterImplementations.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidget.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetBottom.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetHorizontal.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetLeft.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetRight.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetTop.cc \
          src/code/casaqt/QtUtilities/Axis/ExternalAxisWidgetVertical.cc \
          src/code/casaqt/QtUtilities/QtActionGroup.cc \
          src/code/casaqt/QtUtilities/QtButtonGroup.cc \
          src/code/casaqt/QtUtilities/QtComboCheckBox.cc \
          src/code/casaqt/QtUtilities/QtEditingWidget.cc \
          src/code/casaqt/QtUtilities/QtFileDialog.cc \
          src/code/casaqt/QtUtilities/QtIndexChooser.cc \
          src/code/casaqt/QtUtilities/QtLayeredLayout.cc \
          src/code/casaqt/QtUtilities/QtPlotWidget.cc \
          src/code/casaqt/QtUtilities/QtProgressWidget.cc \
          src/code/casaqt/QtUtilities/QtUtilities.cc \
          src/code/casaqt/QwtPlotter/QPAnnotation.cc \
          src/code/casaqt/QwtPlotter/QPAxis.cc \
          src/code/casaqt/QwtPlotter/QPBarPlot.cc \
          src/code/casaqt/QwtPlotter/QPCanvas.cc \
          src/code/casaqt/QwtPlotter/QPCanvasHelpers.cc \
          src/code/casaqt/QwtPlotter/QPData.cc \
          src/code/casaqt/QwtPlotter/QPExporter.cc \
          src/code/casaqt/QwtPlotter/QPFactory.cc \
          src/code/casaqt/QwtPlotter/QPHeaderTableWidget.cc \
          src/code/casaqt/QwtPlotter/QPImageCache.cc \
          src/code/casaqt/QwtPlotter/QPLayeredCanvas.cc \
          src/code/casaqt/QwtPlotter/QPOptions.cc \
          src/code/casaqt/QwtPlotter/QPPanel.cc \
          src/code/casaqt/QwtPlotter/QPPlotItem.cc \
          src/code/casaqt/QwtPlotter/QPPlotter.cc \
          src/code/casaqt/QwtPlotter/QPRasterPlot.cc \
          src/code/casaqt/QwtPlotter/QPScatterPlot.cc \
          src/code/casaqt/QwtPlotter/QPShape.cc \
          src/code/casaqt/QwtPlotter/QPTool.cc \
          casa-source/casatools/src/code/stdcasa/UtilJ.cc \
          casa-source/casatools/src/code/stdcasa/thread/AsynchronousTools.cc \
          casa-source/casatools/src/code/casatools/Config/State.cc \
          casa-source/casatools/src/code/casatools/Proc/Registrar.cc \
          casa-source/casatools/src/code/components/ComponentModels/ComponentList.cc \
          casa-source/casatools/src/code/components/ComponentModels/ComponentShape.cc \
          casa-source/casatools/src/code/components/ComponentModels/ComponentType.cc \
          casa-source/casatools/src/code/components/ComponentModels/ComponentType2.cc \
          casa-source/casatools/src/code/components/ComponentModels/ConstantSpectrum.cc \
          casa-source/casatools/src/code/components/ComponentModels/DiskShape.cc \
          casa-source/casatools/src/code/components/ComponentModels/GaussianShape.cc \
          casa-source/casatools/src/code/components/ComponentModels/LimbDarkenedDiskShape.cc \
          casa-source/casatools/src/code/components/ComponentModels/PointShape.cc \
          casa-source/casatools/src/code/components/ComponentModels/SkyCompBase.cc \
          casa-source/casatools/src/code/components/ComponentModels/SkyCompRep.cc \
          casa-source/casatools/src/code/components/ComponentModels/SkyComponent.cc \
          casa-source/casatools/src/code/components/ComponentModels/SpectralIndex.cc \
          casa-source/casatools/src/code/components/ComponentModels/SpectralModel.cc \
          casa-source/casatools/src/code/components/ComponentModels/TabularSpectrum.cc \
          casa-source/casatools/src/code/components/ComponentModels/TwoSidedShape.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotCanvas.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotCanvasLayout.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotData.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotEvent.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotFactory.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotItem.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotLogger.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotOperation.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotOptions.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/PlotTool.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/Plotter.cc \
          casa-source/casatools/src/code/graphics/GenericPlotter/SimplePlotter.cc \
          casa-source/casatools/src/code/imageanalysis/Utilities/SpectralImageUtil.cc \
          casa-source/casatools/src/code/msvis/MSVis/TransformingVi2.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBufferImpl2.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformBufferImpl.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformDataHandler.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformIterator.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformIteratorFactory.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformManager.cc \
          casa-source/casatools/src/code/mstransform/MSTransform/MSTransformRegridder.cc \
          casa-source/casatools/src/code/mstransform/TVI/ChannelAverageTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/PhaseShiftingTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/DenoisingLib.cc \
          casa-source/casatools/src/code/mstransform/TVI/FreqAxisTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/PolAverageTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/UVContSubTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/UVContSubResult.cc \
          casa-source/casatools/src/code/mstransform/TVI/UtilsTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/PointingInterpolationTVI.cc \
          casa-source/casatools/src/code/mstransform/TVI/SDAtmosphereCorrectionTVI.cc \
          casa-source/casatools/src/code/msvis/MSVis/AsynchronousInterface.cc \
          casa-source/casatools/src/code/msvis/MSVis/AveragingTvi2.cc \
          casa-source/casatools/src/code/msvis/MSVis/AveragingVi2Factory.cc \
          casa-source/casatools/src/code/msvis/MSVis/CalVisBuffer.cc \
          casa-source/casatools/src/code/msvis/MSVis/CalibratingVi2FactoryI.cc \
          casa-source/casatools/src/code/msvis/MSVis/IteratingParameters.cc \
          casa-source/casatools/src/code/msvis/MSVis/LayeredVi2Factory.cc \
          casa-source/casatools/src/code/msvis/MSVis/MSCalEnums.cc \
          casa-source/casatools/src/code/msvis/MSVis/MSIter2.cc \
          casa-source/casatools/src/code/msvis/MSVis/MSUtil.cc \
          casa-source/casatools/src/code/msvis/MSVis/MeasurementSet2.cc \
          casa-source/casatools/src/code/msvis/MSVis/MsRows.cc \
          casa-source/casatools/src/code/msvis/MSVis/PointingDirectionCache.cc \
          casa-source/casatools/src/code/msvis/MSVis/SimpleSimVi2.cc \
          casa-source/casatools/src/code/msvis/MSVis/SpectralWindow.cc \
          casa-source/casatools/src/code/msvis/MSVis/StokesVector.cc \
          casa-source/casatools/src/code/msvis/MSVis/VBContinuumSubtractor.cc \
          casa-source/casatools/src/code/msvis/MSVis/VLAT.cc \
          casa-source/casatools/src/code/msvis/MSVis/Vbi2MsRow.cc \
          casa-source/casatools/src/code/msvis/MSVis/ViColumns2.cc \
          casa-source/casatools/src/code/msvis/MSVis/ViFrequencySelection.cc \
          casa-source/casatools/src/code/msvis/MSVis/ViImplementation2.cc \
          casa-source/casatools/src/code/msvis/MSVis/ViiLayerFactory.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBuffAccumulator.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBuffGroupAcc.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBuffer.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBuffer2.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBufferAsync.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBufferAsyncWrapper.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBufferComponents2.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisBufferUtil.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisImagingWeight.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisModelDataI.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisSet.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisSetUtil.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisibilityIterator.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisibilityIterator2.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImpl.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImpl2.cc \
          casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImplAsync.cc \
          src/code/plotms/apps/casaplotms/casaplotms.cc \
          src/code/plotms/Actions/ActionCache.cc \
          src/code/plotms/Actions/ActionCacheLoad.cc \
          src/code/plotms/Actions/ActionStack.cc \
          src/code/plotms/Actions/ActionStackBack.cc \
          src/code/plotms/Actions/ActionStackBase.cc \
          src/code/plotms/Actions/ActionFlagAll.cc \
          src/code/plotms/Actions/FlagActionUtil.cc \
          src/code/plotms/Actions/ActionAnnotate.cc \
          src/code/plotms/Actions/ActionAnnotateRectangle.cc \
          src/code/plotms/Actions/ActionAnnotateText.cc \
          src/code/plotms/Actions/ActionCacheRelease.cc \
          src/code/plotms/Actions/ActionClearPlotter.cc \
          src/code/plotms/Actions/ActionClearRegions.cc \
          src/code/plotms/Actions/ActionExport.cc \
          src/code/plotms/Actions/ActionExportDialog.cc \
          src/code/plotms/Actions/ActionFactory.cc \
          src/code/plotms/Actions/ActionHoldReleaseDrawing.cc \
          src/code/plotms/Actions/ActionInformation.cc \
          src/code/plotms/Actions/ActionIterFirst.cc \
          src/code/plotms/Actions/ActionIterLast.cc \
          src/code/plotms/Actions/ActionIterNext.cc \
          src/code/plotms/Actions/ActionIterPrev.cc \
          src/code/plotms/Actions/ActionLocate.cc \
          src/code/plotms/Actions/ActionMarkRegions.cc \
          src/code/plotms/Actions/ActionPan.cc \
          src/code/plotms/Actions/ActionPlot.cc \
          src/code/plotms/Actions/ActionQuit.cc \
          src/code/plotms/Actions/ActionSelect.cc \
          src/code/plotms/Actions/ActionSelectFlag.cc \
          src/code/plotms/Actions/ActionSelectUnflag.cc \
          src/code/plotms/Actions/ActionStackForward.cc \
          src/code/plotms/Actions/ActionSubtractRegions.cc \
          src/code/plotms/Actions/ActionSummary.cc \
          src/code/plotms/Actions/ActionSummaryDialog.cc \
          src/code/plotms/Actions/ActionTool.cc \
          src/code/plotms/Actions/ActionTracker.cc \
          src/code/plotms/Actions/ActionTrackerDisplay.cc \
          src/code/plotms/Actions/ActionTrackerHover.cc \
          src/code/plotms/Actions/ActionZoom.cc \
          src/code/plotms/Actions/PlotMSAction.cc \
          src/code/plotms/Client/Client.cc \
          src/code/plotms/Client/ClientFactory.cc \
          src/code/plotms/Client/ClientScript.cc \
          src/code/plotms/Data/CalCache.cc \
          src/code/plotms/Data/MSCache.cc \
          src/code/plotms/Data/MSCacheVolMeter.cc \
          src/code/plotms/Data/PageHeaderCache.cc \
          src/code/plotms/Data/PlotMSAtm.cc \
          src/code/plotms/Data/PlotMSCacheBase.cc \
          src/code/plotms/Data/PlotMSIndexer.cc \
          src/code/plotms/Data/PlotMSVBAverager.cc \
          src/code/plotms/Data/PlotMSCTAverager.cc \
          src/code/plotms/Gui/PlotMSAnnotator.cc \
          src/code/plotms/Gui/PlotMSAveragingWidget.cc \
          src/code/plotms/Gui/PlotMSAxisWidget.cc \
          src/code/plotms/Gui/PlotMSCalibrationWidget.cc \
          src/code/plotms/Gui/PlotMSFlaggingWidget.cc \
          src/code/plotms/Gui/PlotMSLoggerWidget.cc \
          src/code/plotms/Gui/PlotMSPageHeaderDataModel.cc \
          src/code/plotms/Gui/PlotMSPlotter.cc \
          src/code/plotms/Gui/PlotMSSelectionWidget.cc \
          src/code/plotms/Gui/PlotMSSummaryDialog.cc \
          src/code/plotms/Gui/PlotMSTransformationsWidget.cc \
          src/code/plotms/Gui/PlotRangeWidget.cc \
          src/code/plotms/GuiTabs/PlotMSAnnotatorTab.cc \
          src/code/plotms/GuiTabs/PlotMSAxesTab.cc \
          src/code/plotms/GuiTabs/PlotMSCacheTab.cc \
          src/code/plotms/GuiTabs/PlotMSCalibrationTab.cc \
          src/code/plotms/GuiTabs/PlotMSCanvasTab.cc \
          src/code/plotms/GuiTabs/PlotMSDataCollapsible.cc \
          src/code/plotms/GuiTabs/PlotMSDataSummaryTab.cc \
          src/code/plotms/GuiTabs/PlotMSDataSymbolWidget.cc \
          src/code/plotms/GuiTabs/PlotMSDataTab.cc \
          src/code/plotms/GuiTabs/PlotMSDisplayTab.cc \
          src/code/plotms/GuiTabs/PlotMSExportTab.cc \
          src/code/plotms/GuiTabs/PlotMSFlaggingTab.cc \
          src/code/plotms/GuiTabs/PlotMSIterateTab.cc \
          src/code/plotms/GuiTabs/PlotMSOptionsTab.cc \
          src/code/plotms/GuiTabs/PlotMSPlotTab.cc \
          src/code/plotms/GuiTabs/PlotMSTab.cc \
          src/code/plotms/GuiTabs/PlotMSToolsTab.cc \
          src/code/plotms/GuiTabs/PlotMSTransformationsTab.cc \
          src/code/plotms/PlotMS/grpcPlotMSAdaptor.cc \
          src/code/plotms/PlotMS/PlotMS.cc \
          src/code/plotms/PlotMS/PlotMSAveraging.cc \
          src/code/plotms/PlotMS/PlotMSCalibration.cc \
          src/code/plotms/PlotMS/PlotMSConstants.cc \
          src/code/plotms/PlotMS/PlotMSExportParam.cc \
          src/code/plotms/PlotMS/PlotMSFlagging.cc \
          src/code/plotms/PlotMS/PlotMSIterParam.cc \
          src/code/plotms/PlotMS/PlotMSLabelFormat.cc \
          src/code/plotms/PlotMS/PlotMSPageHeaderParam.cc \
          src/code/plotms/PlotMS/PlotMSParameters.cc \
          src/code/plotms/PlotMS/PlotMSRegions.cc \
          src/code/plotms/PlotMS/PlotMSSelection.cc \
          src/code/plotms/PlotMS/PlotMSTransformations.cc \
          src/code/plotms/PlotMS/PlotMSWatchedParameters.cc \
          src/code/plotms/Plots/PlotMSPage.cc \
          src/code/plotms/Plots/PlotMSPages.cc \
          src/code/plotms/Plots/PlotMSPlot.cc \
          src/code/plotms/Plots/PlotMSPlotManager.cc \
          src/code/plotms/Plots/PlotMSPlotParameterGroups.cc \
          src/code/plotms/Plots/PlotMSPlotParameters.cc \
          src/code/plotms/Threads/BackgroundThread.cc \
          src/code/plotms/Threads/CacheThread.cc \
          src/code/plotms/Threads/ExportThread.cc \
          src/code/plotms/Threads/Gui/PlotMSCacheThread.cc \
          src/code/plotms/Threads/Gui/PlotMSDrawThread.cc \
          src/code/plotms/Threads/Gui/PlotMSExportThread.cc \
          src/code/plotms/Threads/Gui/PlotMSThread.cc \
          src/code/plotms/Threads/ThreadController.cc \
          casa-source/casatools/src/code/stdcasa/StdCasa/CasacSupport.cc \
          casa-source/casatools/src/code/stdcasa/StdCasa/record.cc \
          casa-source/casatools/src/code/stdcasa/StdCasa/variant.cc \
          casa-source/casatools/src/code/synthesis/CalLibrary/CalLibraryParse.cc \
          casa-source/casatools/src/code/synthesis/CalLibrary/CalLibraryTools.cc \
          casa-source/casatools/src/code/synthesis/CalTables/BJonesDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/BJonesMBuf.cc \
          casa-source/casatools/src/code/synthesis/CalTables/BJonesMCol.cc \
          casa-source/casatools/src/code/synthesis/CalTables/BJonesTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CLPatchPanel.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTColumns.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTEnums.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTGlobals.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTInterface.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTIter.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTMainColumns.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTMainRecord.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTPatchedInterp.cc \
          src/code/synthesis/CalTables/CTSelection.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTSummary.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CTTimeInterp1.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalBuffer.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalDescBuffer.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalDescColumns.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalDescColumns2.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalDescRecord.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalHistRecord.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalHistoryBuffer.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalHistoryColumns.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalIter.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalMainBuffer.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalMainColumns.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalMainRecord.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalSummary.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalTable2.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalTableDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/CalTableDesc2.cc \
          casa-source/casatools/src/code/synthesis/CalTables/GJonesDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/GJonesMBuf.cc \
          casa-source/casatools/src/code/synthesis/CalTables/GJonesMCol.cc \
          casa-source/casatools/src/code/synthesis/CalTables/GJonesTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/NewCalTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/RIorAParray.cc \
          casa-source/casatools/src/code/synthesis/CalTables/SolvableVJDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/SolvableVJMBuf.cc \
          casa-source/casatools/src/code/synthesis/CalTables/SolvableVJMCol.cc \
          casa-source/casatools/src/code/synthesis/CalTables/SolvableVJTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/TimeVarVJDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/TimeVarVJMBuf.cc \
          casa-source/casatools/src/code/synthesis/CalTables/TimeVarVJMCol.cc \
          casa-source/casatools/src/code/synthesis/CalTables/TimeVarVJTable.cc \
          casa-source/casatools/src/code/synthesis/CalTables/VisJonesDesc.cc \
          casa-source/casatools/src/code/synthesis/CalTables/VisJonesMBuf.cc \
          casa-source/casatools/src/code/synthesis/CalTables/VisJonesMCol.cc \
          casa-source/casatools/src/code/synthesis/CalTables/VisJonesTable.cc \
          casa-source/casatools/src/code/synthesis/ImagerObjects/SynthesisUtilMethods.cc \
          casa-source/casatools/src/code/synthesis/ImagerObjects/SIImageStoreMultiTerm.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/DelayRateFFT.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/XJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/AMueller.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/AccorJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/BPoly.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/CalCorruptor.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/Calibrater.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/CalibratingVi2.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/CalibratingVi2Factory.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/CalSolVi2Organizer.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/ConvFuncDiskCache.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/DJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/EJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/EPJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/EVLASwPow.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/FiltrationTVI.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/FJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/FringeJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/GSpline.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/Jones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/KJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/LJJones.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/MSMetaInfoForCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/Mueller.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SDDoubleCircleFilter.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SDDoubleCircleGainCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SDDoubleCircleGainCalImpl.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SingleDishSkyCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SolvableVisCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SolveDataBuffer.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/StandardVisCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/SteepestDescentSolver.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/TsysGainCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/UVMod.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/VisCal.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/VisCalGlobals.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/VisCalSolver.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/VisCalSolver2.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/VisVector.cc \
          casa-source/casatools/src/code/synthesis/MeasurementComponents/nPBWProjectFT.cc \
          casa-source/casatools/src/code/synthesis/MeasurementEquations/AntennaResponses.cc \
          casa-source/casatools/src/code/synthesis/MeasurementEquations/VPManager.cc \
          casa-source/casatools/src/code/synthesis/MeasurementEquations/VisEquation.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ALMAAperture.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ALMABeamCalcParams.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ALMACalcIlluminationConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ATerm.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/AzElAperture.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/BeamCalc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/BeamCalcParams.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/BeamSkyJones.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/BeamSquint.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/CFBuffer.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/CFCell.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/CFStore.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/CFStore2.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ComponentFTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/ConvolutionFunction.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/FTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/GridFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/HetArrayConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/IlluminationConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/MosaicFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/MosaicFTNew.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/MultiTermFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/MultiTermFTNew.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1D.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DAiry.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DCosPoly.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DEVLA.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DGauss.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DIPoly.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DNumeric.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath1DPoly.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath2D.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMath2DImage.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PBMathInterface.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/PolOuterProduct.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/SetJyGridFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/SimpleComponentFTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/SimplePBConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/SkyJones.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/StokesImageUtil.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/SynthesisError.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/Utils.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/VLACalcIlluminationConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/VPSkyJones.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/VisModelData.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/WPConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/WProjectFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines/rGridFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/SimplePBConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/Utils.cc \
          casa-source/casatools/src/code/synthesis/Utilities/AppRC.cc \
          casa-source/casatools/src/code/synthesis/Utilities/FFT2D.cc \
          casa-source/casatools/src/code/synthesis/Utilities/PointingDirectionCalculator.cc \
          casa-source/casatools/src/code/synthesis/Utilities/PointingDirectionProjector.cc \
          casa-source/casatools/src/code/synthesis/Utilities/SDPosInterpolator.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/VisModelData.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/GridFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/FTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/BriggsCubeWeightor.cc \
          casa-source/casatools/src/code/synthesis/ImagerObjects/SIImageStore.cc \
          casa-source/casatools/src/code/synthesis/ImagerObjects/SDMaskHandler.cc \
          casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageMetaDataConstants.cc \
          casa-source/casatools/src/code/components/ComponentModels/SkyComponentFactory.cc \
          casa-source/casatools/src/code/components/ComponentModels/GaussianDeconvolver.cc \
          casa-source/casatools/src/code/components/ComponentModels/PowerLogPoly.cc \
          casa-source/casatools/casacore/scimath/Mathematics/VectorKernel.cc \
          casa-source/casatools/src/code/imageanalysis/Regions/CasacRegionManager.cc \
          casa-source/casatools/src/code/imageanalysis/IO/RegionTextParser.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnotationBase.cc \
          casa-source/casatools/src/code/imageanalysis/IO/ParameterParser.cc \
          casa-source/casatools/src/code/imageanalysis/IO/AsciiAnnotationFileLine.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnRegion.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnText.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnSymbol.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnVector.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnLine.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnEllipse.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnAnnulus.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnCircle.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnPolygon.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnRotBox.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnCenterBox.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/AnnRectBox.cc \
          casa-source/casatools/casacore/images/Regions/RegionManager.cc \
          casa-source/casatools/casacore/scimath/Mathematics/Geometry.cc \
          casa-source/casatools/src/code/imageanalysis/Annotations/RegionTextList.cc \
          casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageInputProcessor.cc \
          casa-source/casatools/src/code/components/ComponentModels/C11Timer.cc \
          casa-source/casatools/src/code/imageanalysis/ImageAnalysis/CasaImageBeamSet.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/PolOuterProduct.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/CFStore2.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/CFStore.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/CFBuffer.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/SetJyGridFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/MosaicFTNew.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/VPSkyJones.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/HetArrayConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/SimpleComponentFTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/ComponentFTMachine.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/MultiTermFTNew.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/WProjectFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/WPConvFunc.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/MosaicFT.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/BeamSkyJones.cc \
          casa-source/casatools/src/code/synthesis/TransformMachines2/SkyJones.cc \
          casa-source/casatools/src/code/imageanalysis/IO/OutputDestinationChecker.cc \
          sakura-source/libsakura/src/gen_util.cc \
          sakura-source/libsakura/src/mask_edge.cc \
          source/casa-version.cc \
          source/casacore-version.cc \
          source/plotms-version.cc \
          casa-source/casatools/casacore/mirlib/xyio.c \
          casa-source/casatools/casacore/mirlib/headio.c \
          casa-source/casatools/casacore/mirlib/maskio.c \
          casa-source/casatools/casacore/mirlib/hio.c \
          casa-source/casatools/casacore/mirlib/pack.c \
          casa-source/casatools/casacore/mirlib/dio.c \
          casa-source/casatools/casacore/mirlib/bug.c

HEADERS = src/code/plotms/PlotMS/grpcPlotMSAdaptor.qo.h \
src/code/plotms/Gui/PlotMSPlotter.qo.h \
src/code/plotms/GuiTabs/PlotMSAxesTab.qo.h \
src/code/plotms/GuiTabs/PlotMSCacheTab.qo.h \
src/code/casaqt/QtUtilities/QtActionGroup.qo.h \
src/code/casaqt/QtUtilities/QtButtonGroup.qo.h \
src/code/plotms/Gui/PlotMSAveragingWidget.qo.h \
src/code/casaqt/QtUtilities/QtEditingWidget.qo.h \
src/code/plotms/Gui/PlotMSAxisWidget.qo.h \
src/code/plotms/Threads/Gui/PlotMSDrawThread.qo.h \
src/code/plotms/Threads/Gui/PlotMSThread.qo.h \
src/code/plotms/GuiTabs/PlotMSOptionsTab.qo.h \
src/code/plotms/GuiTabs/PlotMSPlotTab.qo.h \
src/code/casaqt/QtUtilities/QtProgressWidget.qo.h \
src/code/casaqt/QwtPlotter/QPPlotter.qo.h \
src/code/casaqt/QtUtilities/QtComboCheckBox.qo.h \
src/code/plotms/GuiTabs/PlotMSDataSummaryTab.qo.h \
src/code/plotms/Gui/PlotMSSummaryDialog.qo.h \
src/code/plotms/Gui/PlotMSFlaggingWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSDataCollapsible.qo.h \
src/code/plotms/Gui/PlotMSSelectionWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSDataSymbolWidget.qo.h \
src/code/plotms/Gui/PlotMSPageHeaderDataModel.qo.h \
src/code/casaqt/QwtPlotter/QPAxis.qo.h \
src/code/casaqt/QwtPlotter/QPCanvasHelpers.qo.h \
src/code/casaqt/QwtPlotter/QPPlotItem.qo.h \
src/code/casaqt/QwtPlotter/QPCanvas.qo.h \
src/code/casaqt/QwtPlotter/QPLayeredCanvas.qo.h \
src/code/casaqt/QwtPlotter/QPPanel.qo.h \
src/code/plotms/GuiTabs/PlotMSTab.qo.h \
src/code/casaqt/QwtPlotter/QPTool.qo.h \
src/code/plotms/Gui/PlotMSTransformationsWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSTransformationsTab.qo.h \
src/code/plotms/Gui/PlotMSCalibrationWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSCalibrationTab.qo.h \
src/code/plotms/Gui/PlotMSLoggerWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSFlaggingTab.qo.h \
src/code/plotms/Threads/Gui/PlotMSCacheThread.qo.h \
src/code/plotms/GuiTabs/PlotMSIterateTab.qo.h \
src/code/plotms/Threads/Gui/PlotMSExportThread.qo.h \
src/code/casaqt/QtUtilities/QtPlotWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSAnnotatorTab.qo.h \
src/code/plotms/GuiTabs/PlotMSDisplayTab.qo.h \
src/code/plotms/Gui/PlotRangeWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSExportTab.qo.h \
src/code/plotms/GuiTabs/PlotMSCanvasTab.qo.h \
src/code/casaqt/QtUtilities/QtIndexChooser.qo.h \
src/code/plotms/GuiTabs/PlotMSToolsTab.qo.h \
src/code/casaqt/QwtPlotter/QPHeaderTableWidget.qo.h \
src/code/plotms/GuiTabs/PlotMSDataTab.qo.h \
src/code/casaqt/QtUtilities/QtFileDialog.qo.h \
src/code/plotms/GuiTabs/PlotMSDisplayTab.qo.h \
src/code/plotms/GuiTabs/PlotMSCanvasTab.qo.h \
src/code/plotms/GuiTabs/PlotMSDataTab.qo.h

RESOURCES += src/code/plotms/Gui/PlotMSPlotter.qrc
RESOURCES += src/code/plotms/GuiTabs/GuiTabs.qrc

FORMS =      src/code/plotms/Gui/PlotMSPlotter.ui \
             src/code/casaqt/QtUtilities/QtProgressCompactWidget.ui \
             src/code/casaqt/QtUtilities/QtProgressWidget.ui \
             src/code/plotms/GuiTabs/PlotMSAnnotatorTab.ui \
             src/code/plotms/GuiTabs/PlotMSFlaggingTab.ui \
             src/code/plotms/Gui/PlotMSFlaggingWidget.ui \
             src/code/casaqt/QtUtilities/PlotFileWidget.ui \
             src/code/casaqt/QtUtilities/PlotLabelWidget.ui \
             src/code/plotms/GuiTabs/PlotMSOptionsTab.ui \
             src/code/plotms/GuiTabs/PlotMSExportTab.ui \
             src/code/plotms/GuiTabs/PlotMSDataSummaryTab.ui \
             src/code/plotms/GuiTabs/PlotMSPlotTab.ui \
             src/code/plotms/GuiTabs/PlotMSToolsTab.ui \
             src/code/casaqt/QtUtilities/PlotColorWidget.ui \
             src/code/casaqt/QtUtilities/PlotFillWidget.ui \
             src/code/casaqt/QtUtilities/PlotLineWidget.ui \
             src/code/casaqt/QtUtilities/PlotSymbolWidget.ui \
             src/code/casaqt/QtUtilities/PlotFontWidget.ui \
             src/code/plotms/GuiTabs/PlotMSDataCollapsible.ui \
             src/code/plotms/GuiTabs/PlotMSAxesTab.ui \
             src/code/plotms/GuiTabs/PlotMSCacheTab.ui \
             src/code/plotms/GuiTabs/PlotMSCalibrationTab.ui \
             src/code/plotms/GuiTabs/PlotMSCanvasTab.ui \
             src/code/plotms/GuiTabs/PlotMSDataTab.ui \
             src/code/plotms/GuiTabs/PlotMSDisplayTab.ui \
             src/code/plotms/GuiTabs/PlotMSIterateTab.ui \
             src/code/plotms/GuiTabs/PlotMSTransformationsTab.ui \
             src/code/plotms/Gui/PlotMSAxisWidget.ui \
             src/code/plotms/Gui/PlotMSCalibrationWidget.ui \
             src/code/plotms/Gui/PlotMSAveragingWidget.ui \
             src/code/casaqt/QtUtilities/QtIndexChooser.ui \
             src/code/plotms/GuiTabs/PlotMSDataSymbolWidget.ui \
             src/code/plotms/Gui/PlotMSTransformationsWidget.ui \
             src/code/plotms/Gui/PlotRangeWidget.ui \
             src/code/plotms/GuiTabs/PlotMSDataSummaryTab.ui \
             src/code/plotms/Gui/PlotMSSummaryDialog.ui \
             src/code/plotms/Gui/PlotMSFlaggingWidget.ui \
             src/code/casaqt/QwtPlotter/QPPlotter.ui

TARGET = casaplotms

python_module.target = py
python_module.commands = scripts/generate-module prefix=build/python.$$QMAKE_HOST.version/casaplotms app=$$APP_EXE protoc=$$PROTOPY shutdown=$$PROTO_SHUTDOWN ping=$$PROTO_PING plotms=$$PROTO_PLOTMS BUILD=$(BUILD)
python_module.depends = $$APP
QMAKE_EXTRA_TARGETS += python_module
