
CONFIG += warn_off

INCLUDEPATH += include include/casagrpc/protos build/include.$$QMAKE_HOST.version src/code casa-source/casatools/src/code casa-source/casatools/casacore 

###
### gRPC libraries and include files are provided with the casatools
### module to ensure compatibility
###
GRPC_LIBS = $$system(scripts/grpc-config --grpc-link)
GRPC_INCLUDE_raw = $$system(scripts/grpc-config --grpc-compile)
GRPC_INCLUDE = $$replace(GRPC_INCLUDE_raw,-I,)
GRPC_PATH = $$system(scripts/grpc-config --grpc-libpath)

system(scripts/do-config)

###
### without this qmake will generate all of the MOC bindings for
### Qt classes that are #ifdefed out...
###
MOC_DIR = build/lib.$$QMAKE_HOST.version/moc
load(moc)
QMAKE_MOC += -DUSE_GRPC

unix:!mac{
  CONFIG += debug cpp11
  exists ( $$[QT_INSTALL_DATA]/mkspecs/features/qwt.prf ) {
    include ( $$[QT_INSTALL_DATA]/mkspecs/features/qwt.prf )
  }
  exists ( $$[QTMAKEFEATURES]/mkspecs/features/qwt.prf ) {
    include ( $$[QTMAKEFEATURES]/mkspecs/features/qwt.prf )
  }
  INCLUDEPATH += $$GRPC_INCLUDE
  INCLUDEPATH += /opt/casa/02/include /usr/include/cfitsio /opt/casa/02/include/qwt
  LIBDIR_LIST = /opt/casa/02/lib /opt/rh/devtoolset-4/root/usr/lib/gcc/x86_64-redhat-linux/5.3.1 /opt/casa/02/lib/gcc5
  GFORTRAN = gfortran
  ###
  ### -lelf is needed simply to cause it to be located from within the casaviewer.app
  ### tree at runtime by the runtime loader... because whatever library depends upon
  ### libelf.so.1 does not include a RPATH/RUNPATH
  ###
  LIBS += -lsqlite3 $$GRPC_LIBS -lpgsbox -lcpgplot -lpgplot -lwcs -lcfitsio -llapack -lgsl -lgslcblas -lblas -lgfortran -lelf -lrt -ldl
  greaterThan(QT_MAJOR_VERSION, 4) {
    LIBDIR_LIST += /usr/lib64
    LIBDIR_LIST += /usr/local/lib
    LIBS += -lqwt-qt5
  }
  QMAKE_CXXFLAGS += -DUSE_GRPC -DCASADATA=\\\"/home/casa/data/trunk\\\"
}

macx:{
  DEFINES += NDEBUG
  CONFIG += cpp11
  #CONFIG += debug cpp11
  MAC_THIRD_PARTY_PATH = "/opt/local"
  exists( /opt/casa/02 ) {
        MAC_THIRD_PARTY_PATH = "/opt/casa/02"
        message("Found /opt/casa/02")
  }
  exists( /opt/casa/03 ) {
        MAC_THIRD_PARTY_PATH = "/opt/casa/03"
        message("Found /opt/casa/03")
  }
  include ( $$[QT_INSTALL_DATA]/mkspecs/features/qwt.prf )
  INCLUDEPATH += $$GRPC_INCLUDE
  INCLUDEPATH += $$MAC_THIRD_PARTY_PATH/include
  LIBDIR_LIST = $$MAC_THIRD_PARTY_PATH/lib $$MAC_THIRD_PARTY_PATH/lib/gcc5
  GFORTRAN = gfortran-mp-5
  LIBS += -lsqlite3 $$GRPC_LIBS -lpgsbox_nox -lcpgplot_nox -lpgplot_nox -lwcs -lcfitsio -llapack -lblas -lgsl -lgfortran
  #QMAKE_CXXFLAGS += -g -O0 -fsanitize=address -fno-omit-frame-pointer -DCASATOOLS -DCASA6
  QMAKE_CXXFLAGS += -DUSE_GRPC
} 

OBJECTS_DIR = build/lib.$$QMAKE_HOST.version/objs
RCC_DIR = build/lib.$$QMAKE_HOST.version/rcc
UI_DIR = build/include.$$QMAKE_HOST.version/ui

CONFIG += object_parallel_to_source

QMAKE_CXXFLAGS += -std=c++11 -DUseCasacoreNamespace -DNO_CRASH_REPORTER -DAIPS_LINUX
QT += xml widgets printsupport

for(p, LIBDIR_LIST) {
    exists($$p):LIBS += -L$$p
    exists($$p):RPATH_LIBS += $$p
}
YACC_DIR = include/yacc
INCLUDEPATH += $$YACC_DIR
QMAKE_CC = ccache $$QMAKE_CC
QMAKE_CXX = ccache $$QMAKE_CXX

unix:!mac{
    # suppress the default RPATH if you wish
    QMAKE_LFLAGS_RPATH=
    # add your own with quoting gyrations to make sure $ORIGIN gets to the command line unexpanded
    QMAKE_LFLAGS += "-Wl,-rpath,\'\$$ORIGIN/../lib\'"
    DESTDIR = build/bin.$$QMAKE_HOST.version/casaviewer.app/usr/bin
    APP = $$DESTDIR/../..
    QMAKE_POST_LINK = scripts/create-app --no-data name=viewer exe=$$DESTDIR/$$TARGET app=$$APP lib=$$DESTDIR/../lib tgt=$$lower($$QMAKE_HOST.os) plugins=$$[QT_INSTALL_PLUGINS] $$GRPC_PATH $$RPATH_LIBS
    APP_EXE = build/bin.$$QMAKE_HOST.version/$${TARGET}-x86_64.AppImage
}

macx:{
    # clang++ required for address sanitizer to work
    #QMAKE_LINK=clang++
    # suppress the default RPATH if you wish
    QMAKE_LFLAGS_RPATH=
    # add your own with quoting gyrations to make sure $ORIGIN gets to the command line unexpanded
    #QMAKE_LFLAGS += "-fsanitize=address -fno-omit-frame-pointer  -Wl,-rpath,\'@loader_path/../Frameworks\'"
    QMAKE_LFLAGS += "-Wl,-rpath,\'@loader_path/../Frameworks\'"
    DESTDIR = build/bin.$$QMAKE_HOST.version
    APP = $$DESTDIR/$${TARGET}.app
    QMAKE_POST_LINK = scripts/create-app --no-data name=viewer exe=$$DESTDIR/$${TARGET}.app/Contents/MacOS/$$TARGET app=$$APP lib=$$DESTDIR/$${TARGET}.app/Contents/Frameworks tgt=$$lower($$QMAKE_HOST.os) plugins=$$[QT_INSTALL_PLUGINS] $$GRPC_PATH $$MAC_THIRD_PARTY_PATH/lib $$RPATH_LIBS
    ICON = casaviewer.icns
    APP_EXE = $$APP
}

fortran.name = Generate .o file for ${QMAKE_FILE_BASE}.f
fortran.input = FORTRAN_SOURCE  # variable containing our input files
fortran.output = ${OBJECTS_DIR}/${QMAKE_FILE_BASE}.o
fortran.commands = $$GFORTRAN -c ${QMAKE_FILE_NAME} -o ${QMAKE_FILE_OUT} -std=legacy
fortran.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += fortran
PRE_TARGETDEPS += compiler_fortran_make_all

yacc_ycc.name = Generate .YCC file for ${QMAKE_FILE_BASE}.yy
yacc_ycc.input = YACC_FILES  # variable containing our input files
yacc_ycc.variable_out = HEADERS
yacc_ycc.commands = bison -y -p ${QMAKE_FILE_BASE} -d -o $$YACC_DIR/${QMAKE_FILE_BASE}.ycc ${QMAKE_FILE_NAME}
yacc_ycc.output = $$YACC_DIR/${QMAKE_FILE_BASE}.ycc
yacc_ycc.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += yacc_ycc
PRE_TARGETDEPS += compiler_yacc_ycc_make_all

lex_lcc.name = Generate .LCC file for ${QMAKE_FILE_BASE}.ll
lex_lcc.input = LEX_FILES  # variable containing our input files
lex_lcc.variable_out = HEADERS
lex_lcc.commands = /usr/bin/flex -P${QMAKE_FILE_BASE} -o$$YACC_DIR/${QMAKE_FILE_BASE}.lcc ${QMAKE_FILE_NAME}
lex_lcc.output = $$YACC_DIR/${QMAKE_FILE_BASE}.lcc
lex_lcc.dependency_type = TYPE_C
QMAKE_EXTRA_COMPILERS += lex_lcc
PRE_TARGETDEPS += compiler_lex_lcc_make_all

PROTO_IMG = casa-source/casatools/src/protos/img.proto
PROTOS = $$PROTO_IMG \
         $$system(scripts/grpc-config --proto-registrar) \
         $$system(scripts/grpc-config --proto-shutdown) \
         $$system(scripts/grpc-config --proto-ping) \

PROTOC = $$system(scripts/grpc-config --grpc-protocpp)
PROTOPY = $$system(scripts/grpc-config --grpc-protopy)
PROTO_SHUTDOWN = $$system(scripts/grpc-config --proto-shutdown)
PROTO_PING = $$system(scripts/grpc-config --proto-ping)
include(grpc_protobuf.pri)

FORTRAN_SOURCE = casa-source/casatools/casacore/scimath_f/fftpak.f casa-source/casatools/casacore/scimath_f/dfftpak.f
YACC_FILES = casa-source/casatools/casacore/tables/TaQL/RecordGram.yy \
  casa-source/casatools/casacore/casa/Json/JsonGram.yy \
  casa-source/casatools/casacore/tables/TaQL/TableGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSStateGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSTimeGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSSpwGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSObservationGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSFieldGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSFeedGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSCorrGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSScanGram.yy \
  casa-source/casatools/casacore/images/Images/ImageExprGram.yy \
  casa-source/casatools/casacore/ms/MSSel/MSArrayGram.yy
LEX_FILES = src/code/display/ds9/ds9lex.ll \
  casa-source/casatools/casacore/tables/TaQL/RecordGram.ll \
  casa-source/casatools/casacore/casa/Json/JsonGram.ll \
  casa-source/casatools/casacore/tables/TaQL/TableGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSStateGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSTimeGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSSpwGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSObservationGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSFieldGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSFeedGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSCorrGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSScanGram.ll \
  casa-source/casatools/casacore/images/Images/ImageExprGram.ll \
  casa-source/casatools/casacore/ms/MSSel/MSArrayGram.ll

SOURCES = src/code/display/apps/casaviewer/casaviewer.cc \
  src/code/display/QtViewer/grpcViewerAdaptor.cc \
  src/code/display/QtViewer/AboutDialogViewer.cc \
  src/code/display/QtViewer/AnimatorHolder.cc \
  src/code/display/QtViewer/AnimatorWidget.cc \
  src/code/display/QtViewer/ColorHistogram.cc \
  src/code/display/QtViewer/ColorHistogramScale.cc \
  src/code/display/QtViewer/ColorTransferWidget.cc \
  src/code/display/QtViewer/CursorTrackingHolder.cc \
  src/code/display/QtViewer/DisplayDataHolder.cc \
  src/code/display/QtViewer/FileBox.cc \
  src/code/display/QtViewer/InActiveDock.cc \
  src/code/display/QtViewer/InvisibleAxis.cc \
  src/code/display/QtViewer/MakeMask.cc \
  src/code/display/QtViewer/MakeRegion.cc \
  src/code/display/QtViewer/ViewerPreferences.cc \
  src/code/display/QtViewer/QtCanvasManager.cc \
  src/code/display/QtViewer/QtCleanPanelGui2.cc \
  src/code/display/QtViewer/QtCleanPanelGui.cc \
  src/code/display/QtViewer/QtDataManager.cc \
  src/code/display/QtViewer/QtDataOptionsPanel.cc \
  src/code/display/QtViewer/QtDisplayData.cc \
  src/code/display/QtViewer/QtDisplayPanel.cc \
  src/code/display/QtViewer/QtDisplayPanelGui.cc \
  src/code/display/QtViewer/QtExportBrowser.cc \
  src/code/display/QtViewer/QtExportManager.cc \
  src/code/display/QtViewer/QtMouseToolBar.cc \
  src/code/display/QtViewer/QtMouseTools.cc \
  src/code/display/QtViewer/QtMouseToolState.cc \
  src/code/display/QtViewer/QtOldMouseTools.cc \
  src/code/display/QtViewer/QtOptions.cc \
  src/code/display/QtViewer/QtPCColorTable.cc \
  src/code/display/QtViewer/QtPixelCanvas.cc \
  src/code/display/QtViewer/QtRegionManager.cc \
  src/code/display/QtViewer/QtViewerBase.cc \
  src/code/display/QtViewer/QtViewer.cc \
  src/code/display/QtViewer/QtViewerPrintGui.cc \
  src/code/display/QtViewer/QtWCBox.cc \
  src/code/display/QtViewer/RegionToolManager.cc \
  src/code/display/DisplayEvents/DTVisible.cc \
  src/code/display/DisplayEvents/PCTool.cc \
  src/code/display/DisplayEvents/MultiPolyToolImpl.cc \
  src/code/display/DisplayEvents/MWCRulerlineTool.cc \
  src/code/display/DisplayEvents/MWCCrosshairTool.cc \
  src/code/display/DisplayEvents/PCPositionEvent.cc \
  src/code/display/DisplayEvents/MWCPolylineTool.cc \
  src/code/display/DisplayEvents/PCInvisTool.cc \
  src/code/display/DisplayEvents/RegionTool.cc \
  src/code/display/DisplayEvents/MultiWCTool.cc \
  src/code/display/DisplayEvents/MWCRectTool.cc \
  src/code/display/DisplayEvents/MWCRTRegion.cc \
  src/code/display/DisplayEvents/PCMotionEvent.cc \
  src/code/display/DisplayEvents/MWCPositionVelocityTool.cc \
  src/code/display/QtViewer/SlicerGen.cc \
  src/code/display/QtViewer/TrackBox.cc \
  \
  casa-source/casatools/src/code/casatools/Config/State.cc \
  casa-source/casatools/src/code/casatools/Proc/Registrar.cc \
  \
  src/code/casaqt/QtUtilities/QtId.cc \
  src/code/display/QtPlotter/Util.cc  \
  src/code/casaqt/QtUtilities/QtPanelBase.cc \
  src/code/display/region/Point.cc \
  src/code/display/region/Ellipse.cc \
  src/code/display/region/Region.cc \
  src/code/display/region/RegionCreator.cc \
  src/code/display/region/QtRegionStats.cc \
  src/code/display/region/QtRegionState.cc \
  src/code/display/QtAutoGui/QtXmlRecord.cc \
  src/code/display/QtAutoGui/QtGuiEntry.cc \
  src/code/display/QtAutoGui/QtAutoGui.cc \
  src/code/display/region/QtRegionDock.cc \
  src/code/guitools/Histogram/ToolTipPicker.cc \
  src/code/guitools/Histogram/BinPlotWidget.cc \
  src/code/guitools/Histogram/FootPrintWidget.cc \
  src/code/display/RegionShapes/QtRegionShapeManager.cc \
  src/code/display/QtPlotter/LegendPreferences.cc \
  src/code/display/QtPlotter/SmoothPreferences.cc \
  src/code/display/QtPlotter/QtProfile.cc \
  src/code/display/QtPlotter/SpecFit.cc \
  src/code/display/QtPlotter/annotations/PropertyListener.cc \
  src/code/display/QtPlotter/annotations/AnnotationText.cc \
  src/code/display/QtPlotter/ProfileFitMarker.cc \
  src/code/display/QtPlotter/QtCanvas.cc \
  src/code/display/QtPlotter/CanvasHolder.cc \
  src/code/display/QtPlotter/LineOverlaysTab.cc \
  src/code/display/QtPlotter/LineOverlaysSearchResultsDialog.cc \
  src/code/display/QtPlotter/SearchRedshiftDialog.cc \
  src/code/display/QtPlotter/SettingsWidget.cc \
  src/code/display/QtPlotter/SpectralPositioningWidget.cc \
  src/code/display/QtViewer/ImageManager/ImageManagerDialog.cc \
  src/code/display/QtViewer/ImageManager/ImageView.cc \
  src/code/display/QtViewer/ImageManager/DisplayLabel.cc \
  src/code/display/QtViewer/ImageManager/DisplayOptionsDialog.cc \
  src/code/display/Fit/RegionBox.cc \
  src/code/display/Fit/Fit2DTool.cc \
  src/code/display/Fit/FindSourcesDialog.cc \
  src/code/display/Fit/PixelRangeDialog.cc \
  src/code/display/Fit/ResidualHistogramDialog.cc \
  src/code/display/Fit/Fit2DLogDialog.cc \
  src/code/display/Fit/Gaussian2DFitter.cc \
  src/code/display/Fit/ColorComboDelegate.cc \
  src/code/display/Fit/ComponentListWrapper.cc \
  src/code/display/Slicer/SliceStatisticsFactory.cc \
  src/code/display/Slicer/SegmentTracer.cc \
  src/code/display/Slicer/SlicePlot.cc \
  src/code/display/Slicer/SlicerMainWindow.cc \
  src/code/display/Slicer/ImageSlice.cc \
  src/code/display/Slicer/SliceColorPreferences.cc \
  src/code/display/Slicer/SlicePlotPreferences.cc \
  src/code/display/Slicer/SliceZoomer.cc \
  src/code/display/Slicer/SliceSegment.cc \
  src/code/display/Slicer/SliceAxisDraw.cc \
  src/code/display/Slicer/SliceStatistics.cc \
  src/code/display/Slicer/ImageSliceColorBar.cc \
  src/code/display/Slicer/SliceWorker.cc \
  src/code/display/Slicer/SliceStatisticsDistance.cc \
  src/code/display/Slicer/SliceStatisticsPosition.cc \
  src/code/guitools/Histogram/HistogramMain.cc \
  src/code/guitools/Histogram/ImageLoader.cc \
  src/code/guitools/Histogram/SaveHistogramWidget.cc \
  src/code/display/Clean/CleanGui.cc \
  src/code/display/region/Polygon.cc \
  src/code/display/region/HistogramTab.cc \
  src/code/display/ds9/ds9lex.cc \
  src/code/display/ds9/ds9writer.cc \
  src/code/display/ds9/ds9parse.cc \
  src/code/display/ds9/ds9parser.cc \
  src/code/display/ds9/ds9context.cc \
  src/code/guitools/Histogram/Histogram.cc \
  src/code/guitools/Histogram/FitWidget.cc \
  src/code/guitools/Histogram/RangeControlsWidget.cc \
  src/code/guitools/Histogram/BinCountWidget.cc \
  src/code/guitools/Histogram/ChannelRangeWidget.cc \
  src/code/guitools/Histogram/ZoomWidget.cc \
  src/code/display/RegionShapes/QtSingleRegionShape.cc \
  src/code/display/RegionShapes/QtNewRegionShape.cc \
  src/code/display/RegionShapes/RegionShapes.cc \
  src/code/display/RegionShapes/RegionShape.cc \
  src/code/display/RegionShapes/RSFileReaderWriter.cc \
  src/code/display/RegionShapes/DS9FileReader.cc \
  src/code/display/RegionShapes/DS9FileWriter.cc \
  src/code/display/RegionShapes/XMLFileReaderWriter.cc \
  src/code/display/RegionShapes/RSUtils.cc \
  src/code/display/QtPlotter/ColorSummaryDelegate.cc \
  src/code/display/QtPlotter/SearchMoleculesWidget.cc \
  src/code/display/QtPlotter/SearchMoleculesResultsWidget.cc \
  src/code/display/QtPlotter/SpecFitSettingsWidgetOptical.cc \
  src/code/display/QtPlotter/SpecFitSettingsWidgetRadio.cc \
  src/code/display/QtPlotter/GaussianEstimateDialog.cc \
  src/code/display/QtPlotter/SearchMoleculesDialog.cc \
  src/code/display/QtPlotter/MomentSettingsWidgetOptical.cc \
  src/code/display/QtPlotter/MomentSettingsWidgetRadio.cc \
  src/code/display/QtViewer/ImageManager/ImageScroll.cc \
  src/code/guitools/Histogram/ColorPreferences.cc \
  src/code/guitools/Histogram/RangePicker.cc \
  src/code/guitools/Histogram/FitterPoisson.cc \
  src/code/guitools/Histogram/ColorDelegate.cc \
  src/code/guitools/Histogram/FitterGaussian.cc \
  src/code/guitools/Histogram/HistogramMarkerPoisson.cc \
  src/code/guitools/Histogram/HistogramMarkerGaussian.cc \
  src/code/guitools/Histogram/Fitter.cc \
  src/code/display/region/HistogramGraph.cc \
  src/code/display/Display/Options.cc \
  src/code/display/Display/DisplayEnums.cc \
  src/code/display/Display/ColorConversion.cc \
  src/code/display/Display/ColormapDefinition.cc \
  src/code/display/Display/DParameterMapKeyChoice.cc \
  src/code/display/Display/DParameterColorChoice.cc \
  src/code/display/Display/DParameterChoice.cc \
  src/code/display/Display/DisplayParameter.cc \
  src/code/display/Display/DParameterString.cc \
  src/code/display/Display/PanelDisplay.cc \
  src/code/display/Display/PCVGBuffer.cc \
  src/code/display/Display/PixelCanvas.cc \
  src/code/display/Display/PixelCanvasColorTable.cc \
  src/code/display/Display/WCPGFilter.cc \
  src/code/display/Display/WorldCanvas.cc \
  src/code/display/Display/WorldCanvasHolder.cc \
  src/code/display/Display/WorldCanvasPGPlotDriver.cc \
  src/code/display/Display/Attribute.cc \
  src/code/display/Display/AttributeBuffer.cc \
  src/code/display/Display/AttValBase.cc \
  src/code/display/Display/ColormapManager.cc \
  src/code/display/Display/DParameterButton.cc \
  src/code/display/DisplayEvents/WorldCanvasEvent.cc \
  src/code/display/DisplayEvents/PCRefreshEvent.cc \
  src/code/display/DisplayEvents/MWCEllipseTool.cc \
  src/code/display/DisplayEvents/DisplayEvent.cc \
  src/code/display/DisplayEvents/WCRefreshEH.cc \
  src/code/display/DisplayEvents/WCRefreshEvent.cc \
  src/code/display/DisplayEvents/MWCRTZoomer.cc \
  src/code/display/DisplayEvents/PixelCanvasEvent.cc \
  src/code/display/DisplayEvents/MWCETRegion.cc \
  src/code/display/DisplayEvents/MWCPTRegion.cc \
  src/code/display/DisplayEvents/MWCPolyTool.cc \
  src/code/display/DisplayEvents/PCMotionEH.cc \
  src/code/display/DisplayEvents/PCPositionEH.cc \
  src/code/display/DisplayEvents/PCRefreshEH.cc \
  src/code/display/DisplayEvents/WCPositionEH.cc \
  src/code/display/DisplayEvents/DisplayTool.cc \
  src/code/display/DisplayEvents/PCITFiddler.cc \
  src/code/display/DisplayDatas/TblAsXYDM.cc \
  src/code/display/DisplayDatas/WorldAxesDM.cc \
  src/code/display/DisplayDatas/WedgeDM.cc \
  src/code/display/DisplayDatas/LatticePADD2.cc \
  src/code/display/DisplayDatas/LattPADMVector2.cc \
  src/code/display/DisplayDatas/ScrollingRasterDM.cc \
  src/code/display/DisplayDatas/LatticeAsVector2.cc \
  src/code/display/DisplayDatas/AxesDisplayData.cc \
  src/code/display/DisplayDatas/TblAsRasterDM.cc \
  src/code/display/DisplayDatas/DisplayData.cc \
  src/code/display/DisplayDatas/ActiveCaching2dDD.cc \
  src/code/display/DisplayDatas/Histogram2dDD.cc \
  src/code/display/DisplayDatas/PassiveCachingDD.cc \
  src/code/display/DisplayDatas/PassiveTableDD.cc \
  src/code/display/DisplayDatas/PrincipalAxesDD.cc \
  src/code/display/DisplayDatas/ScrollingRasterDD.cc \
  src/code/display/DisplayDatas/SkyCatOverlayDD.cc \
  src/code/display/DisplayDatas/TblAsRasterDD.cc \
  src/code/display/DisplayDatas/TblAsXYDD.cc \
  src/code/display/DisplayDatas/WedgeDD.cc \
  src/code/display/DisplayDatas/WorldAxesDD.cc \
  src/code/display/DisplayDatas/DisplayMethod.cc \
  src/code/display/DisplayDatas/Histogram2dDM.cc \
  src/code/display/DisplayDatas/PrincipalAxesDM.cc \
  src/code/display/DisplayDatas/SkyCatOverlayDM.cc \
  src/code/display/DisplayDatas/AxesDisplayMethod.cc \
  src/code/display/DisplayDatas/CachingDisplayData.cc \
  src/code/display/DisplayDatas/CachingDisplayMethod.cc \
  src/code/display/QtPlotter/ColorSummaryWidget.cc \
  src/code/display/QtPlotter/ProfileTaskMonitor.cc \
  src/code/display/QtPlotter/SpecFitLogDialog.cc \
  src/code/display/QtPlotter/SpecFitSettingsFixedTableCell.cc \
  src/code/display/QtPlotter/GaussianEstimateWidget.cc \
  src/code/display/QtPlotter/ThresholdingBinPlotDialog.cc \
  src/code/display/QtPlotter/QtProfilePrefs.cc \
  src/code/display/QtPlotter/WorldCanvasTranslator.cc \
  src/code/display/QtPlotter/SmoothSpinBox.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeFactory.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeChannel.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeZoom.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeAnnotation.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeContextMenu.cc \
  src/code/display/QtPlotter/canvasMode/CanvasModeRangeSelection.cc \
  src/code/display/QtPlotter/SpecFitPolynomial.cc \
  src/code/display/QtPlotter/SpecFitGaussian.cc \
  src/code/display/QtPlotter/QtPlotSettings.cc \
  src/code/display/QtPlotter/MolecularLine.cc \
  src/code/display/QtPlotter/QtMWCTools.cc \
  src/code/display/QtPlotter/CanvasCurve.cc \
  src/code/display/QtPlotter/annotations/Annotation.cc \
  src/code/display/QtPlotter/canvasMode/CanvasMode.cc \
  src/code/display/QtPlotter/QtOldMWCTools.cc \
  src/code/display/QtPlotter/annotations/AnnotationEditor.cc \
  src/code/display/QtPlotter/annotations/AnnotationEditorText.cc \
  src/code/display/QtPlotter/conversion/ConverterChannel.cc \
  src/code/display/QtPlotter/conversion/ConverterFrequency.cc \
  src/code/display/QtPlotter/conversion/ConverterFrequencyVelocity.cc \
  src/code/display/QtPlotter/conversion/ConverterFrequencyWavelength.cc \
  src/code/display/QtPlotter/conversion/ConverterWavelength.cc \
  src/code/display/QtPlotter/conversion/ConverterWavelengthVelocity.cc \
  src/code/display/QtPlotter/conversion/ConverterWavelengthFrequency.cc \
  src/code/display/QtPlotter/conversion/ConverterVelocity.cc \
  src/code/display/QtPlotter/conversion/ConverterVelocityFrequency.cc \
  src/code/display/QtPlotter/conversion/ConverterVelocityWavelength.cc \
  src/code/display/QtPlotter/conversion/ConverterIntensity.cc \
  src/code/display/QtPlotter/SearchMoleculesResultDisplayer.cc \
  src/code/display/QtPlotter/ProfileTaskFacilitator.cc \
  src/code/display/QtPlotter/conversion/Converter.cc \
  src/code/display/DisplayEvents/MultiEllipseTool.cc \
  src/code/display/DisplayEvents/MultiPointTool.cc \
  src/code/display/DisplayEvents/MultiRectTool.cc \
  src/code/display/DisplayEvents/MultiPolyTool.cc \
  src/code/display/DisplayEvents/MultiPolylineTool.cc \
  src/code/display/DisplayEvents/MultiPVTool.cc \
  src/code/display/DisplayEvents/MultiPVToolImpl.cc \
  src/code/display/DisplayEvents/MultiEllipseToolImpl.cc \
  src/code/display/DisplayEvents/MultiPointToolImpl.cc \
  src/code/display/DisplayEvents/MultiRectToolImpl.cc \
  src/code/display/DisplayEvents/MultiPolylineToolImpl.cc \
  src/code/display/DisplayEvents/WCMotionEH.cc \
  src/code/display/DisplayEvents/MWCPannerTool.cc \
  src/code/display/DisplayEvents/WCMotionEvent.cc \
  src/code/display/DisplayEvents/WCPositionEvent.cc \
  src/code/display/region/PVLine.cc \
  src/code/display/region/Rectangle.cc \
  src/code/display/region/Polyline.cc \
  src/code/display/region/RegionSource.cc \
  src/code/display/region/QtRegionSource.cc \
  src/code/display/region/QtRegionSourceFactory.cc \
  src/code/display/Display/DisplayMarkerDefinitions.cc \
  src/code/display/Display/DisplayCoordinateSystem.cc \
  src/code/display/Display/MultiWCHolder.cc \
  src/code/display/Display/ColormapInfo.cc \
  src/code/display/Display/DisplayState.cc \
  src/code/display/Display/Colormap.cc \
  casa-source/casatools/casacore/measures/Measures/MCRadialVelocity.cc \
  casa-source/casatools/casacore/measures/Measures/MCEpoch.cc \
  casa-source/casatools/casacore/measures/Measures/MCBase.cc \
  casa-source/casatools/casacore/measures/Measures/MCBaseline.cc \
  casa-source/casatools/casacore/measures/Measures/MRBase.cc \
  casa-source/casatools/casacore/measures/Measures/Aberration.cc \
  casa-source/casatools/casacore/measures/Measures/MeasComet.cc \
  casa-source/casatools/casacore/measures/Measures/MCFrame.cc \
  casa-source/casatools/casacore/measures/Measures/MEarthMagnetic.cc \
  casa-source/casatools/casacore/measures/Measures/MPosition.cc \
  casa-source/casatools/casacore/measures/Measures/MRadialVelocity.cc \
  casa-source/casatools/casacore/casa/Arrays/Slice.cc \
  casa-source/casatools/casacore/casa/Arrays/Slicer.cc \
  casa-source/casatools/casacore/casa/Arrays/Vector_tmpl.cc \
  casa-source/casatools/casacore/casa/Arrays/Array_tmpl.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/CoordinateSystem.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/LinearXform.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/LinearXform2.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/DirectionCoordinate.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/Direction2Coordinate.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/FITSCoordinateUtil.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/Spectral2Coordinate.cc \
  casa-source/casatools/src/code/stdcasa/UtilJ.cc \
  casa-source/casatools/src/code/stdcasa/thread/AsynchronousTools.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageMetaDataConstants.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageMask.cc \
  casa-source/casatools/src/code/imageanalysis/IO/ProfileFitterEstimatesFileParser.cc \
  casa-source/casatools/src/code/imageanalysis/IO/OutputDestinationChecker.cc \
  casa-source/casatools/casacore/casa/IO/IPositionIO.cc \
  casa-source/casatools/src/code/imageanalysis/Regions/CasacRegionManager.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImagePadder.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/PVGenerator.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/PixelValueManipulatorData.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/CasaImageBeamSet.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageProfileFitter.cc \
  casa-source/casatools/src/code/imageanalysis/IO/FitterEstimatesFileParser.cc \
  casa-source/casatools/src/code/imageanalysis/IO/AsciiAnnotationFileLine.cc \
  casa-source/casatools/src/code/imageanalysis/IO/RegionTextParser.cc \
  casa-source/casatools/src/code/imageanalysis/IO/ParameterParser.cc \
  casa-source/casatools/src/code/imageanalysis/IO/ImageProfileFitterResults.cc \
  casa-source/casatools/src/code/imageanalysis/IO/LogFile.cc \
  casa-source/casatools/casacore/images/Images/Image_tmpl.cc \
  casa-source/casatools/src/code/imageanalysis/Images/ComponentListImage.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnVector.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnLine.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnText.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnCircle.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnRotBox.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/RegionTextList.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnRectBox.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnEllipse.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnPolygon.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnPolyline.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnRegion.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnSymbol.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnotationBase.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnAnnulus.cc \
  casa-source/casatools/src/code/imageanalysis/Annotations/AnnCenterBox.cc \
  casa-source/casatools/casacore/measures/Measures/SolarPos.cc \
  casa-source/casatools/casacore/measures/Measures/MBaseline.cc \
  casa-source/casatools/casacore/measures/Measures/MDirection.cc \
  casa-source/casatools/casacore/measures/Measures/MFrequency.cc \
  casa-source/casatools/casacore/measures/Measures/MDoppler.cc \
  casa-source/casatools/casacore/measures/Measures/MCDoppler.cc \
  casa-source/casatools/casacore/measures/Measures/MCDirection.cc \
  casa-source/casatools/casacore/measures/Measures/MCFrequency.cc \
  casa-source/casatools/casacore/measures/Measures/MeasTable.cc \
  casa-source/casatools/casacore/measures/Measures/MCPosition.cc \
  casa-source/casatools/casacore/measures/Measures/MeasFrame.cc \
  casa-source/casatools/casacore/measures/Measures/MeasTableMul.cc \
  casa-source/casatools/casacore/measures/Measures/MeasIERS.cc \
  casa-source/casatools/casacore/measures/Measures/MeasJPL.cc \
  casa-source/casatools/casacore/measures/Measures/MeasMath.cc \
  casa-source/casatools/casacore/measures/Measures/Quality.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasType.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableQuantumDesc.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasRefDesc.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasValueDesc.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasDescBase.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasOffsetDesc.cc \
  casa-source/casatools/casacore/measures/TableMeasures/TableMeasColumn.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/QualityCoordinate.cc \
  casa-source/casatools/casacore/tables/Tables/ScaRecordColDesc.cc \
  casa-source/casatools/casacore/tables/Tables/ScaRecordColData.cc \
  casa-source/casatools/casacore/tables/Tables/ColumnsIndex.cc \
  casa-source/casatools/casacore/tables/Tables/NullTable.cc \
  casa-source/casatools/casacore/tables/Tables/TableLocker.cc \
  casa-source/casatools/casacore/tables/Tables/TableLock.cc \
  casa-source/casatools/casacore/tables/Tables/TableKeyword.cc \
  casa-source/casatools/casacore/tables/Tables/TableInfo.cc \
  casa-source/casatools/casacore/tables/Tables/ArrColData.cc \
  casa-source/casatools/casacore/tables/Tables/ArrColDesc.cc \
  casa-source/casatools/casacore/tables/Tables/ArrColDesc_tmpl.cc \
  casa-source/casatools/casacore/tables/Tables/RowNumbers.cc \
  casa-source/casatools/casacore/tables/Tables/ArrayColumnBase.cc \
  casa-source/casatools/casacore/tables/Tables/RowCopier.cc \
  casa-source/casatools/casacore/tables/Tables/TableUtil.cc \
  casa-source/casatools/casacore/casa/Quanta/QC.cc \
  casa-source/casatools/casacore/casa/Quanta/Unit.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap2.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap3.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap4.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap5.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap6.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitMap7.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitName.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitDim.cc \
  casa-source/casatools/casacore/casa/Quanta/QBase.cc \
  casa-source/casatools/casacore/casa/Quanta/MVuvw.cc \
  casa-source/casatools/casacore/casa/Quanta/MVAngle.cc \
  casa-source/casatools/casacore/casa/Quanta/QLogical2.cc \
  casa-source/casatools/casacore/casa/Quanta/MVBaseline.cc \
  casa-source/casatools/casacore/casa/Quanta/MVEarthMagnetic.cc \
  casa-source/casatools/casacore/casa/Quanta/MVDirection.cc \
  casa-source/casatools/casacore/casa/Quanta/MVFrequency.cc \
  casa-source/casatools/casacore/casa/Quanta/MVPosition.cc \
  casa-source/casatools/casacore/casa/Quanta/MVTime.cc \
  casa-source/casatools/casacore/casa/Quanta/MVDoppler.cc \
  casa-source/casatools/casacore/casa/Quanta/UnitVal.cc \
  casa-source/casatools/casacore/casa/Quanta/MeasValue.cc \
  casa-source/casatools/casacore/casa/Quanta/Quantum2.cc \
  casa-source/casatools/casacore/casa/Quanta/QuantumHolder.cc \
  casa-source/casatools/casacore/casa/Quanta/MVRadialVelocity.cc \
  casa-source/casatools/casacore/casa/Quanta/RotMatrix.cc \
  casa-source/casatools/casacore/casa/Quanta/Euler.cc \
  casa-source/casatools/casacore/casa/Quanta/MVEpoch.cc \
  casa-source/casatools/casacore/casa/BasicSL/String.cc \
  casa-source/casatools/casacore/casa/BasicMath/Random.cc \
  casa-source/casatools/casacore/casa/BasicMath/Math.cc \
  casa-source/casatools/casacore/casa/BasicSL/Complex.cc \
  casa-source/casatools/casacore/casa/BasicSL/IComplex.cc \
  casa-source/casatools/casacore/casa/Utilities/Compare.cc \
  casa-source/casatools/casacore/casa/Utilities/ValType.cc \
  casa-source/casatools/casacore/casa/Utilities/CountedPtr2.cc \
  casa-source/casatools/casacore/casa/Utilities/Sort.cc \
  casa-source/casatools/casacore/casa/Utilities/SortError.cc \
  casa-source/casatools/casacore/casa/Utilities/Regex.cc \
  casa-source/casatools/casacore/casa/Utilities/MUString.cc \
  casa-source/casatools/casacore/casa/Utilities/StringDistance.cc \
  casa-source/casatools/casacore/casa/Utilities/RecordTransformable.cc \
  casa-source/casatools/casacore/casa/Utilities/Copy2.cc \
  casa-source/casatools/casacore/casa/Utilities/Precision.cc \
  casa-source/casatools/casacore/casa/Utilities/DataType.cc \
  casa-source/casatools/casacore/casa/Exceptions/Error2.cc \
  casa-source/casatools/casacore/casa/Arrays/ArrayUtil2.cc \
  casa-source/casatools/casacore/casa/Arrays/Matrix_tmpl.cc \
  casa-source/casatools/casacore/casa/Arrays/ArrayError.cc \
  casa-source/casatools/casacore/casa/Arrays/ArrayBase.cc \
  casa-source/casatools/casacore/casa/Arrays/ArrayPosIter.cc \
  casa-source/casatools/casacore/casa/Arrays/IPosition.cc \
  casa-source/casatools/casacore/casa/Arrays/AxesMapping.cc \
  casa-source/casatools/casacore/casa/Arrays/AxesSpecifier.cc \
  casa-source/casatools/casacore/casa/Arrays/ExtendSpecifier.cc \
  casa-source/casatools/casacore/casa/Containers/Allocator.cc \
  casa-source/casatools/casacore/casa/Containers/Block.cc \
  casa-source/casatools/casacore/casa/Containers/Block_tmpl.cc \
  casa-source/casatools/casacore/casa/Containers/ValueHolderRep.cc \
  casa-source/casatools/casacore/casa/Containers/RecordRep.cc \
  casa-source/casatools/casacore/scimath/StatsFramework/BiweightStatisticsData.cc \
  casa-source/casatools/casacore/scimath/StatsFramework/StatisticsData.cc \
  casa-source/casatools/casacore/scimath/StatsFramework/ClassicalStatisticsData.cc \
  casa-source/casatools/casacore/scimath/Fitting/LSQFit.cc \
  casa-source/casatools/casacore/scimath/Fitting/LSQFit3.cc \
  casa-source/casatools/casacore/scimath/Fitting/LSQMatrix.cc \
  casa-source/casatools/casacore/scimath/Fitting/LSQMatrix2.cc \
  casa-source/casatools/casacore/tables/DataMan/VirtArrCol.cc \
  casa-source/casatools/casacore/tables/DataMan/DataManagerColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/StManColumnBase.cc \
  casa-source/casatools/casacore/tables/Tables/RefColumn.cc \
  casa-source/casatools/casacore/tables/Tables/RefTable.cc \
  casa-source/casatools/casacore/tables/Tables/Table.cc \
  casa-source/casatools/casacore/tables/TaQL/UDFBase.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseUpdate.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseSortKey.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseQuery.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseUtil.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseJoin.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseTableList.cc \
  casa-source/casatools/casacore/tables/TaQL/RecordExpr.cc \
  casa-source/casatools/casacore/tables/TaQL/TableGram.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeRecord.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeRep.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprDerNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprDerNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprLogicNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprLogicNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprFuncNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprGroupAggrFunc.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprGroupAggrFuncArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprUDFNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprUDFNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprMathNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprMathNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeSet.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeSetElem.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeSetOpt.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeUtil.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseFunc.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLJoin.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseGroupby.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParseProject.cc \
  casa-source/casatools/casacore/tables/DataMan/MemoryStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMCoordColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMCubeBuff.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMIdColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMDataColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledShapeStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledDataStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledFileHelper.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledFileAccess.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/VirtualTaQLColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledStManAccessor.cc \
  casa-source/casatools/casacore/tables/DataMan/DataManAccessor.cc \
  casa-source/casatools/casacore/tables/Tables/TableAttr.cc \
  casa-source/casatools/casacore/tables/Tables/TableCopy.cc \
  casa-source/casatools/casacore/tables/Tables/ScaColDesc_tmpl.cc \
  casa-source/casatools/casacore/tables/Tables/ReadAsciiTable.cc \
  casa-source/casatools/casacore/tables/Tables/TableDesc.cc \
  casa-source/casatools/casacore/tables/Tables/BaseTabIter.cc \
  casa-source/casatools/casacore/tables/Tables/SetupNewTab.cc \
  casa-source/casatools/casacore/tables/Tables/BaseTable.cc \
  casa-source/casatools/casacore/tables/Tables/BaseColumn.cc \
  casa-source/casatools/casacore/tables/Tables/ColumnDesc.cc \
  casa-source/casatools/casacore/tables/Tables/TableTrace.cc \
  casa-source/casatools/casacore/tables/Tables/TableError.cc \
  casa-source/casatools/casacore/tables/Tables/TableRow.cc \
  casa-source/casatools/casacore/tables/Tables/TableColumn.cc \
  casa-source/casatools/casacore/tables/Tables/ColumnSet.cc \
  casa-source/casatools/casacore/tables/Tables/RefRows.cc \
  casa-source/casatools/casacore/tables/Tables/PlainTable.cc \
  casa-source/casatools/casacore/tables/Tables/PlainColumn.cc \
  casa-source/casatools/casacore/tables/Tables/TableLockData.cc \
  casa-source/casatools/casacore/tables/Tables/TableSyncData.cc \
  casa-source/casatools/casacore/tables/Tables/BaseColDesc.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMDirColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMIndColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMIndStringColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMStringHandler.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMShape.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMIndex.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMOption.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMCube.cc \
  casa-source/casatools/casacore/tables/DataMan/StArrAipsIO.cc \
  casa-source/casatools/casacore/tables/DataMan/StIndArrAIO.cc \
  casa-source/casatools/casacore/tables/DataMan/StArrayFile.cc \
  casa-source/casatools/casacore/tables/DataMan/StIndArray.cc \
  casa-source/casatools/casacore/tables/DataMan/SSMBase.cc \
  casa-source/casatools/casacore/tables/DataMan/IncrementalStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/MSMIndColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/MSMDirColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/MSMColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/MSMBase.cc \
  casa-source/casatools/casacore/tables/DataMan/StManAipsIO.cc \
  casa-source/casatools/casacore/tables/DataMan/ForwardCol.cc \
  casa-source/casatools/casacore/tables/DataMan/BitFlagsEngine.cc \
  casa-source/casatools/casacore/tables/DataMan/DataManError.cc \
  casa-source/casatools/casacore/tables/DataMan/StManColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/CompressFloat.cc \
  casa-source/casatools/casacore/tables/DataMan/CompressComplex.cc \
  casa-source/casatools/casacore/tables/DataMan/StandardStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/StandardStManAccessor.cc \
  casa-source/casatools/casacore/tables/DataMan/IncrStManAccessor.cc \
  casa-source/casatools/casacore/tables/DataMan/ISMBase.cc \
  casa-source/casatools/casacore/tables/Tables/TableRecord.cc \
  casa-source/casatools/casacore/tables/DataMan/DataManager.cc \
  casa-source/casatools/casacore/tables/Tables/ArrayColumn_tmpl.cc \
  casa-source/casatools/casacore/tables/Tables/ScalarColumn_tmpl.cc \
  casa-source/casatools/casacore/tables/Tables/ConcatColumn.cc \
  casa-source/casatools/casacore/tables/Tables/ConcatTable.cc \
  casa-source/casatools/casacore/tables/Tables/ConcatRows.cc \
  casa-source/casatools/casacore/tables/Tables/ColumnCache.cc \
  casa-source/casatools/casacore/tables/Tables/SubTabDesc.cc \
  casa-source/casatools/casacore/tables/Tables/ColDescSet.cc \
  casa-source/casatools/casacore/tables/Tables/TableCache.cc \
  casa-source/casatools/casacore/tables/Tables/StorageOption.cc \
  casa-source/casatools/casacore/tables/Tables/TableIter.cc \
  casa-source/casatools/casacore/tables/Tables/TableIterProxy.cc \
  casa-source/casatools/casacore/tables/Tables/TabPath.cc \
  casa-source/casatools/casacore/tables/Tables/TableRecordRep.cc \
  casa-source/casatools/casacore/tables/Tables/MemoryTable.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLShow.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprFuncNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLNodeVisitor.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLStyle.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLNodeDer.cc \
  casa-source/casatools/casacore/tables/TaQL/RecordGram.cc \
  casa-source/casatools/casacore/tables/TaQL/TableParse.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprUnitNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprGroup.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprConeNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprRange.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLResult.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprNode.cc \
  casa-source/casatools/casacore/tables/TaQL/MArrayBase.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLNodeHandler.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprAggrNode.cc \
  casa-source/casatools/casacore/tables/TaQL/ExprAggrNodeArray.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLNode.cc \
  casa-source/casatools/casacore/tables/TaQL/TaQLNodeRep.cc \
  casa-source/casatools/casacore/tables/LogTables/NewFile.cc \
  casa-source/casatools/casacore/tables/LogTables/LoggerHolder.cc \
  casa-source/casatools/casacore/tables/LogTables/TableLogSink.cc \
  casa-source/casatools/casacore/tables/Tables/TableProxy.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMFile.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/TSMCubeMMap.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledColumnStMan.cc \
  casa-source/casatools/casacore/tables/DataMan/DataManInfo.cc \
  casa-source/casatools/casacore/tables/DataMan/VirtColEng.cc \
  casa-source/casatools/casacore/tables/DataMan/ISMBucket.cc \
  casa-source/casatools/casacore/tables/DataMan/ISMColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/ISMIndColumn.cc \
  casa-source/casatools/casacore/tables/DataMan/ISMIndex.cc \
  casa-source/casatools/casacore/tables/DataMan/TiledCellStMan.cc \
  casa-source/casatools/src/code/asdmstman/AsdmColumn.cc \
  casa-source/casatools/src/code/asdmstman/AsdmIndex.cc \
  casa-source/casatools/src/code/asdmstman/AsdmStMan.cc \
  casa-source/casatools/src/code/asdmstman/Register.cc \
  src/code/display/DisplayDatas/MSAsRaster.cc \
  src/code/display/Utilities/DlTarget.cc \
  src/code/display/Utilities/DlHandle.cc \
  src/code/display/Utilities/dtor.cc \
  src/code/display/Utilities/ImageProperties.cc \
  src/code/display/Utilities/DisplayOptions.cc \
  src/code/display/Utilities/Lowlevel.cc \
  casa-source/casatools/casacore/casa/System/PGPlotter.cc \
  casa-source/casatools/casacore/casa/System/PGPlotterInterface.cc \
  casa-source/casatools/casacore/casa/System/PGPlotterNull.cc \
  casa-source/casatools/casacore/casa/System/Aipsrc.cc \
  casa-source/casatools/casacore/casa/System/Casarc.cc \
  casa-source/casatools/casacore/casa/System/AipsrcValue2.cc \
  casa-source/casatools/casacore/casa/System/AipsrcVString.cc \
  casa-source/casatools/casacore/casa/System/AipsrcBool.cc \
  casa-source/casatools/casacore/casa/System/ProgressMeter.cc \
  casa-source/casatools/casacore/casa/System/ObjectID.cc \
  casa-source/casatools/casacore/casa/System/ObjectID2.cc \
  casa-source/casatools/casacore/ms/MSOper/MSDerivedValues.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSDopplerUtil.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSTableImpl.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSWeatherColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSTable.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSysCalColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSStateColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSpWindowColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSpectralWindow.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFeed.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFieldColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFlagCmdColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFreqOffColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSHistoryColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSAntennaColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSDataDescColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSDataDescription.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSDopplerColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSPointingColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSObsColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSPolColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSProcessorColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSDoppler.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSField.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFlagCmd.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFreqOffset.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSHistory.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSObservation.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSPointing.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSPolarization.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSProcessor.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSource.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSState.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSysCal.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSWeather.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSourceIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSTableIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSParse.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSSourceColumns.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSSpwErrorHandler.cc \
  casa-source/casatools/casacore/ms/MSSel/MSObservationParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSDataDescIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFieldIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSPolIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSArrayParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSCorrParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFeedIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFieldParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSPolnParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSScanParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSpwIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSStateIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSTimeParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSObservationGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFieldGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFeedGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSCorrGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSScanGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSPolnGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSpwGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSUvDistGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSStateGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSTimeGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSArrayGram.cc \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaIndex.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSelectionErrorHandler.cc \
  casa-source/casatools/casacore/ms/MSSel/MSAntennaParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSelection.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSpwParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSFeedParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSStateParse.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSelectableTable.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSelectionTools.cc \
  casa-source/casatools/casacore/ms/MSSel/MSSelectionError.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSMainColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSFeedColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSAntenna.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MeasurementSet.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSColumns.cc \
  casa-source/casatools/casacore/ms/MeasurementSets/MSIter.cc \
  casa-source/casatools/casacore/measures/Measures/Stokes.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/StokesCoordinate.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/SpectralCoordinate.cc \
  casa-source/casatools/casacore/measures/Measures/VelocityMachine.cc \
  casa-source/casatools/casacore/measures/Measures/Precession.cc \
  casa-source/casatools/casacore/measures/Measures/Muvw.cc \
  casa-source/casatools/casacore/measures/Measures/Nutation.cc \
  casa-source/casatools/casacore/measures/Measures/Measure.cc \
  casa-source/casatools/casacore/measures/Measures/MeasData.cc \
  casa-source/casatools/casacore/measures/Measures/MConvertBase.cc \
  casa-source/casatools/casacore/measures/Measures/MeasureHolder.cc \
  casa-source/casatools/casacore/measures/Measures/MEpoch.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/Coordinate.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/ObsInfo.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/Projection.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/CoordinateUtil.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/LinearCoordinate.cc \
  casa-source/casatools/casacore/coordinates/Coordinates/TabularCoordinate.cc \
  casa-source/casatools/casacore/lattices/Lattices/Lattices_tmpl.cc \
  casa-source/casatools/casacore/lattices/Lattices/LatticeStepper.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCIntersection.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCLELMask.cc \
  casa-source/casatools/casacore/lattices/LEL/LELCoordinates.cc \
  casa-source/casatools/casacore/lattices/Lattices/TiledLineStepper.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LatticeHistProgress.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LatticeHistSpecialize.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LattStatsProgress.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/Fit2D.cc \
  casa-source/casatools/casacore/lattices/Lattices/TiledShape.cc \
  casa-source/casatools/casacore/lattices/Lattices/TileStepper.cc \
  casa-source/casatools/casacore/lattices/Lattices/LatticeBase.cc \
  casa-source/casatools/casacore/lattices/Lattices/LatticeLocker.cc \
  casa-source/casatools/casacore/lattices/LEL/LELRegion.cc \
  casa-source/casatools/casacore/lattices/LEL/LELAttribute.cc \
  casa-source/casatools/casacore/lattices/LEL/LELBinary2.cc \
  casa-source/casatools/casacore/lattices/LEL/LELFunction2.cc \
  casa-source/casatools/casacore/lattices/LEL/LatticeExprNode.cc \
  casa-source/casatools/casacore/lattices/LEL/LELArrayBase.cc \
  casa-source/casatools/casacore/lattices/LEL/LELLattCoord.cc \
  casa-source/casatools/casacore/lattices/LEL/LELLattCoordBase.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LattStatsSpecialize.cc \
  casa-source/casatools/casacore/lattices/LEL/LELUnary2.cc \
  casa-source/casatools/casacore/lattices/LRegions/RegionType.cc \
  casa-source/casatools/casacore/lattices/LRegions/LattRegionHolder.cc \
  casa-source/casatools/casacore/lattices/LRegions/FITSMask.cc \
  casa-source/casatools/casacore/lattices/LRegions/LatticeRegion.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCExtension.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCEllipsoid.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCRegionFixed.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCRegionMulti.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCRegion.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCRegion2.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCComplement.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCDifference.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCPagedMask.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCHDF5Mask.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCPixelSet.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCSlicer.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCBox.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCMask.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCUnion.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCPolygon.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCStretch.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCConcatenation.cc \
  casa-source/casatools/casacore/lattices/LRegions/LCRegionSingle.cc \
  casa-source/casatools/casacore/lattices/Lattices/LatticeIndexer.cc \
  casa-source/casatools/casacore/lattices/Lattices/LatticeNavigator.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LatticeProgress.cc \
  casa-source/casatools/casacore/lattices/LatticeMath/LatticeStatsBase.cc \
  casa-source/casatools/casacore/lattices/Lattices/PixelCurve1D.cc \
  casa-source/casatools/casacore/casa/Logging/LogFilterInterface.cc \
  casa-source/casatools/casacore/casa/Logging/MemoryLogSink.cc \
  casa-source/casatools/casacore/casa/Logging/NullLogSink.cc \
  casa-source/casatools/casacore/casa/Logging/LogFilter.cc \
  casa-source/casatools/casacore/casa/Logging/LogSinkInterface.cc \
  casa-source/casatools/casacore/casa/Logging/LogSink.cc \
  casa-source/casatools/casacore/casa/Logging/LogIO.cc \
  casa-source/casatools/casacore/casa/Logging/LogMessage.cc \
  casa-source/casatools/casacore/casa/Logging/StreamLogSink.cc \
  casa-source/casatools/casacore/casa/Logging/LogOrigin.cc \
  casa-source/casatools/casacore/casa/Json/JsonError.cc \
  casa-source/casatools/casacore/casa/Json/JsonOut.cc \
  casa-source/casatools/casacore/casa/Json/JsonKVMap.cc \
  casa-source/casatools/casacore/casa/Json/JsonValue.cc \
  casa-source/casatools/casacore/casa/Json/JsonParser.cc \
  casa-source/casatools/casacore/images/Images/ImageExprGram.cc \
  casa-source/casatools/casacore/images/Images/ImageInfo.cc \
  casa-source/casatools/casacore/images/Images/FITSErrorImage.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrGroup.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrGroupCasa.cc \
  casa-source/casatools/casacore/images/Images/PagedImage2.cc \
  casa-source/casatools/casacore/images/Images/ImageOpener.cc \
  casa-source/casatools/casacore/images/Images/ImageExprParse.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrHandler.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrHandlerCasa.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrHandlerHDF5.cc \
  casa-source/casatools/casacore/images/Images/ImageAttrGroupHDF5.cc \
  casa-source/casatools/casacore/images/Images/ImageFITS2Converter.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/SpectralCollapser.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageCollapserData.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageCollapser2.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/SpectralFitter.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageFactory2.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ProfileFitResults.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageInputProcessor.cc \
  casa-source/casatools/src/code/imageanalysis/ImageAnalysis/ImageMomentsProgress.cc \
  casa-source/casatools/casacore/images/Regions/ImageRegion.cc \
  casa-source/casatools/casacore/images/Images/FITSImgParser.cc \
  casa-source/casatools/casacore/images/Regions/RegionHandlerMemory.cc \
  casa-source/casatools/casacore/images/Regions/RegionHandler.cc \
  casa-source/casatools/casacore/images/Regions/RegionHandlerHDF5.cc \
  casa-source/casatools/casacore/images/Regions/RegionHandlerTable.cc \
  casa-source/casatools/casacore/images/Regions/WCEllipsoid.cc \
  casa-source/casatools/casacore/images/Regions/WCExtension.cc \
  casa-source/casatools/casacore/images/Regions/WCCompound.cc \
  casa-source/casatools/casacore/images/Regions/WCComplement.cc \
  casa-source/casatools/casacore/images/Regions/WCDifference.cc \
  casa-source/casatools/casacore/images/Regions/RegionManager.cc \
  casa-source/casatools/casacore/images/Regions/WCRegion.cc \
  casa-source/casatools/casacore/images/Regions/WCRegion2.cc \
  casa-source/casatools/casacore/images/Regions/WCUnion.cc \
  casa-source/casatools/casacore/images/Regions/WCPolygon.cc \
  casa-source/casatools/casacore/images/Regions/WCLELMask.cc \
  casa-source/casatools/casacore/images/Regions/WCIntersection.cc \
  casa-source/casatools/casacore/images/Regions/WCConcatenation.cc \
  casa-source/casatools/casacore/images/Regions/WCBox.cc \
  casa-source/casatools/casacore/images/Images/MIRIADImage.cc \
  casa-source/casatools/casacore/images/Images/ImageBeamSet.cc \
  casa-source/casatools/casacore/images/Images/LELImageCoord.cc \
  casa-source/casatools/casacore/images/Images/HDF5Image2.cc \
  casa-source/casatools/casacore/images/Images/FITSQualityMask.cc \
  ./casa-source/casatools/casacore/images/Images/ImageUtilities.cc \
  casa-source/casatools/casacore/casa/OS/Timer.cc \
  casa-source/casatools/casacore/casa/OS/SymLink.cc \
  casa-source/casatools/casacore/casa/OS/Path.cc \
  casa-source/casatools/casacore/casa/OS/EnvVar.cc \
  casa-source/casatools/casacore/casa/OS/DynLib.cc \
  casa-source/casatools/casacore/casa/OS/Conversion.cc \
  casa-source/casatools/casacore/casa/OS/LECanonicalConversion.cc \
  casa-source/casatools/casacore/casa/OS/MemoryTrace.cc \
  casa-source/casatools/casacore/casa/OS/RegularFile.cc \
  casa-source/casatools/casacore/casa/OS/Directory.cc \
  casa-source/casatools/casacore/casa/OS/DOos.cc \
  casa-source/casatools/casacore/casa/OS/DirectoryIterator.cc \
  casa-source/casatools/casacore/casa/OS/File.cc \
  casa-source/casatools/casacore/casa/OS/HostInfo.cc \
  casa-source/casatools/casacore/casa/OS/CanonicalConversion.cc \
  casa-source/casatools/casacore/casa/OS/Time.cc \
  casa-source/casatools/casacore/casa/IO/TypeIO.cc \
  casa-source/casatools/casacore/casa/IO/FileUnbufferedIO.cc \
  casa-source/casatools/casacore/casa/IO/BucketBase.cc \
  casa-source/casatools/casacore/casa/IO/BucketMapped.cc \
  casa-source/casatools/casacore/casa/IO/BucketBuffered.cc \
  casa-source/casatools/casacore/casa/IO/MultiHDF5.cc \
  casa-source/casatools/casacore/casa/IO/MultiFile.cc \
  casa-source/casatools/casacore/casa/IO/MemoryIO.cc \
  casa-source/casatools/casacore/casa/IO/MMapfdIO.cc \
  casa-source/casatools/casacore/casa/IO/MFFileIO.cc \
  casa-source/casatools/casacore/casa/IO/BucketFile.cc \
  casa-source/casatools/casacore/casa/IO/FiledesIO.cc \
  casa-source/casatools/casacore/casa/IO/MultiFileBase.cc \
  casa-source/casatools/casacore/casa/IO/RegularFileIO.cc \
  casa-source/casatools/casacore/casa/IO/LockFile.cc \
  casa-source/casatools/casacore/casa/IO/FilebufIO.cc \
  casa-source/casatools/casacore/casa/IO/FileLocker.cc \
  casa-source/casatools/casacore/casa/IO/ByteIO.cc \
  casa-source/casatools/casacore/casa/IO/CanonicalIO.cc \
  casa-source/casatools/casacore/casa/IO/LECanonicalIO.cc \
  casa-source/casatools/casacore/casa/IO/BucketCache.cc \
  casa-source/casatools/casacore/casa/OS/OMP.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5DataSet.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5Object.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5Record.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5DataType.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5Group.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5Error.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5File.cc \
  casa-source/casatools/casacore/casa/HDF5/HDF5HidMeta.cc \
  casa-source/casatools/casacore/casa/Containers/Record2Interface.cc \
  casa-source/casatools/casacore/casa/Containers/ValueHolder.cc \
  casa-source/casatools/casacore/casa/Containers/RecordInterface.cc \
  casa-source/casatools/casacore/casa/Containers/RecordDescRep.cc \
  casa-source/casatools/casacore/casa/Containers/RecordDesc.cc \
  casa-source/casatools/casacore/casa/Containers/IterError.cc \
  casa-source/casatools/casacore/casa/Containers/Record.cc \
  casa-source/casatools/casacore/casa/Containers/Record2.cc \
  casa-source/casatools/casacore/fits/FITS/FITSHistoryUtil.cc \
  casa-source/casatools/casacore/fits/FITS/BinTable.cc \
  casa-source/casatools/casacore/fits/FITS/FITSDateUtil.cc \
  casa-source/casatools/casacore/fits/FITS/hdu2.cc \
  casa-source/casatools/casacore/fits/FITS/fits.cc \
  casa-source/casatools/casacore/fits/FITS/fitsio.cc \
  casa-source/casatools/casacore/fits/FITS/blockio.cc \
  casa-source/casatools/casacore/fits/FITS/FITSError.cc \
  casa-source/casatools/casacore/fits/FITS/FITSSpectralUtil.cc \
  casa-source/casatools/casacore/fits/FITS/FITSKeywordUtil.cc \
  casa-source/casatools/casacore/fits/FITS/FITSTable2.cc \
  casa-source/casatools/casacore/images/Images/FITSImage.cc \
  casa-source/casatools/casacore/images/Images/FITSQualityImage.cc \
  casa-source/casatools/casacore/scimath/Mathematics/NumericTraits.cc \
  casa-source/casatools/casacore/scimath/Mathematics/GaussianBeam.cc \
  casa-source/casatools/casacore/scimath/Mathematics/VectorKernel.cc \
  casa-source/casatools/casacore/scimath/Mathematics/Interpolate2D.cc \
  casa-source/casatools/casacore/scimath/Mathematics/Combinatorics.cc \
  casa-source/casatools/casacore/scimath/Mathematics/FFTServer.cc \
  casa-source/casatools/casacore/scimath/Mathematics/FFTPack.cc \
  casa-source/casatools/casacore/scimath/Mathematics/FFTW.cc \
  casa-source/casatools/casacore/scimath/Mathematics/Geometry.cc \
  casa-source/casatools/casacore/scimath/StatsFramework/ZScoreCalculator.cc \
  casa-source/casatools/casacore/scimath/Functionals/FuncExpression.cc \
  casa-source/casatools/casacore/scimath/Functionals/FuncExprData.cc \
  casa-source/casatools/casacore/casa/BasicSL/Constants.cc \
  \
  casa-source/casatools/src/code/msvis/MSVis/VLAT.cc \
  casa-source/casatools/src/code/msvis/MSVis/MsRows.cc \
  casa-source/casatools/src/code/msvis/MSVis/MeasurementSet2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBufferAsync.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisibilityIterator.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisibilityIterator2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImpl.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImpl2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisibilityIteratorImplAsync.cc \
  casa-source/casatools/src/code/msvis/MSVis/AsynchronousInterface.cc \
  casa-source/casatools/src/code/msvis/MSVis/ViFrequencySelection.cc \
  casa-source/casatools/src/code/msvis/MSVis/ViImplementation2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBufferAsyncWrapper.cc \
  casa-source/casatools/src/code/msvis/MSVis/PointingDirectionCache.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisModelDataI.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBuffer.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisImagingWeight.cc \
  casa-source/casatools/src/code/msvis/MSVis/SpectralWindow.cc \
  casa-source/casatools/src/code/msvis/MSVis/ViColumns2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBufferImpl2.cc \
  casa-source/casatools/src/code/msvis/MSVis/Vbi2MsRow.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBufferComponents2.cc \
    casa-source/casatools/src/code/msvis/MSVis/MSIter2.cc \
  casa-source/casatools/src/code/msvis/MSVis/VisBuffer2.cc \
  casa-source/casatools/src/code/msvis/MSVis/MSUtil.cc \
  \
  casa-source/casatools/casacore/mirlib/bug.c \
  casa-source/casatools/casacore/mirlib/dio.c \
  casa-source/casatools/casacore/mirlib/headio.c \
  casa-source/casatools/casacore/mirlib/hio.c \
  casa-source/casatools/casacore/mirlib/key.c \
  casa-source/casatools/casacore/mirlib/maskio.c \
  casa-source/casatools/casacore/mirlib/pack.c \
  casa-source/casatools/casacore/mirlib/scrio.c \
  casa-source/casatools/casacore/mirlib/uvio.c \
  casa-source/casatools/casacore/mirlib/xyio.c \
  casa-source/casatools/casacore/mirlib/xyzio.c \
  casa-source/casatools/casacore/casa/IO/AipsIO.cc \
  casa-source/casatools/casacore/casa/System/Choice.cc \
  casa-source/casatools/casacore/casa/System/AppInfo.cc \
  casa-source/casatools/casacore/casa/System/AppState.cc \
  casa-source/casatools/casacore/casa/Arrays/ArrayPartMath.cc \
  casa-source/casatools/casacore/casa/Arrays/Array2.cc \
  casa-source/casatools/casacore/casa/Arrays/Array2Math.cc \
  casa-source/casatools/casacore/casa/Arrays/Matrix2Math.cc \
  \
  casa-source/casatools/src/code/components/ComponentModels/DiskShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/C11Timer.cc \
  casa-source/casatools/src/code/components/ComponentModels/LimbDarkenedDiskShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/SkyComponentFactory.cc \
  casa-source/casatools/src/code/components/ComponentModels/TabularSpectrum.cc \
  casa-source/casatools/src/code/components/ComponentModels/ConstantSpectrum.cc \
  casa-source/casatools/src/code/components/ComponentModels/SpectralIndex.cc \
  casa-source/casatools/src/code/components/ComponentModels/TwoSidedShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/SpectralModel.cc \
  casa-source/casatools/src/code/components/ComponentModels/GaussianShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/SkyComponent.cc \
  casa-source/casatools/src/code/components/ComponentModels/SkyCompRep.cc \
  casa-source/casatools/src/code/components/ComponentModels/ComponentList.cc \
  casa-source/casatools/src/code/components/ComponentModels/ComponentShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/PointShape.cc \
  casa-source/casatools/src/code/components/ComponentModels/PowerLogPoly.cc \
  casa-source/casatools/src/code/components/ComponentModels/SkyCompBase.cc \
  casa-source/casatools/src/code/components/ComponentModels/ComponentType.cc \
  casa-source/casatools/src/code/components/ComponentModels/ComponentType2.cc \
  casa-source/casatools/src/code/components/ComponentModels/GaussianDeconvolver.cc \
  casa-source/casatools/src/code/components/SpectralComponents/LogTransformedPolynomialSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/PowerLogPolynomialSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/GaussianMultipletSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/PolynomialSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/LorentzianSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/GaussianSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/CompiledSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralElementFactory.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralEstimate.cc \
  casa-source/casatools/src/code/components/SpectralComponents/PCFSpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralList.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralFit.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralElement.cc \
  casa-source/casatools/src/code/components/SpectralComponents/SpectralElement2.cc \
  casa-source/casatools/src/code/stdcasa/StdCasa/CasacSupport.cc \
  casa-source/casatools/src/code/stdcasa/StdCasa/record.cc \
  casa-source/casatools/src/code/stdcasa/StdCasa/variant.cc \
  src/code/display/DisplayCanvas/DefaultWCSizeControlHandler.cc \
  src/code/display/DisplayCanvas/DefaultWCCoordinateHandler.cc \
  src/code/display/DisplayCanvas/WCResampleHandler.cc \
  src/code/display/DisplayCanvas/WCSimpleResampleHandler.cc \
  src/code/display/DisplayCanvas/WCSizeControlHandler.cc \
  src/code/display/DisplayCanvas/WCLinearScaleHandler.cc \
  src/code/display/DisplayCanvas/WCPowerScaleHandler.cc \
  src/code/display/DisplayCanvas/WCDataScaleHandler.cc \
  src/code/display/DisplayCanvas/WCCSNLAxisLabeller.cc \
  src/code/display/DisplayCanvas/WCCSAxisLabeller.cc \
  src/code/display/DisplayCanvas/WCAxisLabeller.cc \
  src/code/display/DisplayCanvas/WCCoordinateHandler.cc \
  casa-source/casatools/src/code/spectrallines/Splatalogue/Searcher.cc \
  casa-source/casatools/src/code/spectrallines/Splatalogue/SearcherFactory.cc \
  casa-source/casatools/src/code/spectrallines/Splatalogue/SplatResult.cc \
  casa-source/casatools/src/code/spectrallines/Splatalogue/SQLiteSearch/DatabaseConnector.cc \
  casa-source/casatools/src/code/spectrallines/Splatalogue/SQLiteSearch/SearcherSQLite.cc \
  \
  include/casagrpc/types/variant.cc \
  include/casagrpc/types/record.cc \
  \
  source/casa-version.cc \
  source/casacore-version.cc

HEADERS = src/code/display/QtViewer/DisplayDataHolder.h \
  src/code/display/QtViewer/grpcViewerAdaptor.qo.h \
  src/code/display/QtViewer/QtLELEdit.qo.h \
  src/code/display/QtViewer/InvisibleAxis.h \
  src/code/display/QtViewer/QtOptions.h \
  src/code/display/QtViewer/QtPCColorTable.h \
  src/code/display/QtViewer/QtWCBox.h \
  src/code/display/QtViewer/AboutDialogViewer.qo.h \
  src/code/display/QtViewer/AnimatorHolder.qo.h \
  src/code/display/QtViewer/AnimatorWidget.qo.h \
  src/code/display/QtViewer/ColorHistogram.qo.h \
  src/code/display/QtViewer/ColorHistogramScale.qo.h \
  src/code/display/QtViewer/ColorTransferWidget.qo.h \
  src/code/display/QtViewer/CursorTrackingHolder.qo.h \
  src/code/display/QtViewer/FileBox.qo.h \
  src/code/display/QtViewer/InActiveDock.qo.h \
  src/code/display/QtViewer/MakeMask.qo.h \
  src/code/display/QtViewer/MakeRegion.qo.h \
  src/code/display/QtViewer/ViewerPreferences.qo.h \
  src/code/display/QtViewer/QtCanvasManager.qo.h \
  src/code/display/QtViewer/QtCleanPanelGui2.qo.h \
  src/code/display/QtViewer/QtCleanPanelGui.qo.h \
  src/code/display/QtViewer/QtDataManager.qo.h \
  src/code/display/QtViewer/QtDataOptionsPanel.qo.h \
  src/code/display/QtViewer/QtDisplayData.qo.h \
  src/code/display/QtViewer/QtDisplayPanel.qo.h \
  src/code/display/QtViewer/QtDisplayPanelGui.qo.h \
  src/code/display/QtViewer/QtExportBrowser.qo.h \
  src/code/display/QtViewer/QtExportManager.qo.h \
  src/code/display/QtViewer/QtMouseToolBar.qo.h \
  src/code/display/QtViewer/QtMouseTools.qo.h \
  src/code/display/QtViewer/QtMouseToolState.qo.h \
  src/code/display/QtViewer/QtOldMouseTools.qo.h \
  src/code/display/QtPlotter/QtOldMWCTools.qo.h \
  src/code/display/QtViewer/QtPixelCanvas.qo.h \
  src/code/display/QtViewer/QtRegionManager.qo.h \
  src/code/display/QtViewer/QtViewerBase.qo.h \
  src/code/display/QtViewer/QtViewer.qo.h \
  src/code/display/QtViewer/QtViewerPrintGui.qo.h \
  src/code/display/QtViewer/RegionToolManager.qo.h \
  src/code/display/QtViewer/SlicerGen.qo.h \
  src/code/display/QtViewer/TrackBox.qo.h \
  src/code/display/QtViewer/QtDisplayDataGui.qo.h \
  \
  src/code/display/region/Polyline.qo.h \
  src/code/display/region/Region.qo.h \
  src/code/display/region/PVLine.qo.h \
  src/code/display/region/QtRegionSource.qo.h \
  src/code/display/region/QtRegionState.qo.h \
  src/code/display/region/QtRegionDock.qo.h \
  src/code/display/region/QtRegionStats.qo.h \
  src/code/guitools/Histogram/BinPlotWidget.qo.h \
  src/code/guitools/Histogram/FootPrintWidget.qo.h \
  src/code/display/RegionShapes/QtRegionShapeManager.qo.h \
  src/code/display/RegionShapes/RSUtils.qo.h \
  src/code/display/QtPlotter/LegendPreferences.qo.h \
  src/code/display/QtPlotter/SmoothPreferences.qo.h \
  src/code/display/QtPlotter/QtProfile.qo.h \
  src/code/display/QtPlotter/QtCanvas.qo.h \
  src/code/display/QtPlotter/CanvasHolder.qo.h \
  src/code/display/QtPlotter/LineOverlaysTab.qo.h \
  src/code/display/QtPlotter/LineOverlaysSearchResultsDialog.qo.h \
  src/code/display/QtPlotter/SearchRedshiftDialog.qo.h \
  src/code/display/QtPlotter/SettingsWidget.qo.h \
  src/code/display/QtPlotter/SpectralPositioningWidget.qo.h \
  src/code/display/QtViewer/ImageManager/ImageManagerDialog.qo.h \
  src/code/display/QtViewer/ImageManager/ImageView.qo.h \
  src/code/display/QtViewer/ImageManager/DisplayLabel.qo.h \
  src/code/display/Fit/Fit2DTool.qo.h \
  src/code/display/Fit/FindSourcesDialog.qo.h \
  src/code/display/Fit/PixelRangeDialog.qo.h \
  src/code/display/Fit/ResidualHistogramDialog.qo.h \
  src/code/display/Fit/Fit2DLogDialog.qo.h \
  src/code/display/Slicer/SlicePlot.qo.h \
  src/code/display/Slicer/SlicerMainWindow.qo.h \
  src/code/display/Slicer/ImageSlice.qo.h \
  src/code/guitools/Histogram/HistogramMain.qo.h \
  src/code/guitools/Histogram/ImageLoader.qo.h \
  src/code/guitools/Histogram/SaveHistogramWidget.qo.h \
  src/code/display/Clean/CleanGui.qo.h \
  src/code/display/region/HistogramTab.qo.h \
  src/code/guitools/Histogram/FitWidget.qo.h \
  src/code/guitools/Histogram/RangeControlsWidget.qo.h \
  src/code/guitools/Histogram/BinCountWidget.qo.h \
  src/code/guitools/Histogram/ChannelRangeWidget.qo.h \
  src/code/guitools/Histogram/ZoomWidget.qo.h \
  src/code/display/RegionShapes/QtSingleRegionShape.qo.h \
  src/code/display/RegionShapes/QtNewRegionShape.qo.h \
  src/code/display/RegionShapes/DS9FileWriter.qo.h \
  src/code/display/QtPlotter/SearchMoleculesWidget.qo.h \
  src/code/display/QtPlotter/SearchMoleculesResultsWidget.qo.h \
  src/code/display/QtPlotter/SpecFitSettingsWidgetOptical.qo.h \
  src/code/display/QtPlotter/SpecFitSettingsWidgetRadio.qo.h \
  src/code/display/QtPlotter/GaussianEstimateDialog.qo.h \
  src/code/display/QtPlotter/SearchMoleculesDialog.qo.h \
  src/code/display/QtPlotter/MomentSettingsWidgetOptical.qo.h \
  src/code/display/QtPlotter/MomentSettingsWidgetRadio.qo.h \
  src/code/display/QtViewer/ImageManager/ImageScroll.qo.h \
  src/code/display/Slicer/SliceColorPreferences.qo.h \
  src/code/display/Slicer/SlicePlotPreferences.qo.h \
  src/code/display/Slicer/SliceSegment.qo.h \
  src/code/guitools/Histogram/ColorPreferences.qo.h \
  src/code/display/region/HistogramGraph.qo.h \
  src/code/display/QtPlotter/SpecFitLogDialog.qo.h \
  src/code/display/QtPlotter/SpecFitSettingsFixedTableCell.qo.h \
  src/code/display/QtPlotter/GaussianEstimateWidget.qo.h \
  src/code/display/QtPlotter/ThresholdingBinPlotDialog.qo.h \
  src/code/display/QtPlotter/QtMWCTools.qo.h \
  src/code/display/QtPlotter/annotations/AnnotationEditor.qo.h \
  src/code/display/QtPlotter/annotations/AnnotationEditorText.qo.h \
  src/code/display/QtPlotter/QtProfilePrefs.qo.h \
  src/code/display/QtPlotter/ColorSummaryWidget.qo.h \
  src/code/casaqt/QtUtilities/QtPanelBase.qo.h \
  src/code/display/QtAutoGui/QtAutoGui.qo.h \
  src/code/display/QtAutoGui/QtGuiEntry.qo.h

RESOURCES += src/code/display/QtViewer/QtViewer.qrc \
  src/code/display/region/QtRegionDock.qrc \
  src/code/display/QtAutoGui/QtAutoGui.qrc \
  src/code/display/QtPlotter/QtPlotter.qrc

FORMS = src/code/display/region/QtRegionState.ui \
  src/code/display/region/QtImageRegionStats.ui \
  src/code/display/region/QtMsRegionStats.ui \
  src/code/display/region/SlicerStats.ui \
  src/code/display/region/PVLineStats.ui \
  src/code/display/region/QtRegionDock.ui \
  src/code/display/region/QtRegionStats.ui \
  src/code/display/QtViewer/AboutDialogViewer.ui \
  src/code/display/QtViewer/AnimatorHolder.ui \
  src/code/display/QtViewer/AnimatorWidget.ui \
  src/code/display/QtViewer/ColorHistogram.ui \
  src/code/guitools/Histogram/BinPlotWidget.ui \
  src/code/guitools/Histogram/FootPrintWidget.ui \
  src/code/display/QtViewer/ColorTransferWidget.ui \
  src/code/display/QtViewer/CursorTrackingHolder.ui \
  src/code/display/QtAutoGui/adjustmentBottom.ui \
  src/code/display/QtAutoGui/sliderLabelItem.ui \
  src/code/display/QtAutoGui/sliderEditorItem.ui \
  src/code/display/QtAutoGui/lineEditorItem.ui \
  src/code/display/QtAutoGui/comboItem.ui \
  src/code/display/QtAutoGui/checkButtonItem.ui \
  src/code/display/QtAutoGui/pushButtonItem.ui \
  src/code/display/QtAutoGui/minMaxEditorItem.ui \
  src/code/display/QtAutoGui/pairItem.ui \
  src/code/display/QtViewer/QtCanvasManager.ui \
  src/code/display/QtViewer/CursorLink.ui \
  src/code/display/QtViewer/SlicerGen.ui \
  src/code/display/QtViewer/QtDataManager.ui \
  src/code/display/QtViewer/QtDataMgrMsSelect.ui \
  src/code/display/QtViewer/VOParam.ui \
  src/code/display/QtViewer/QtDataOptionsPanel.ui \
  src/code/display/RegionShapes/QtRegionShapeManager.ui \
  src/code/display/RegionShapes/QtRSFileLoader.ui \
  src/code/display/RegionShapes/QtRSFileSaver.ui \
  src/code/display/QtViewer/printControl.ui \
  src/code/display/QtViewer/QtRegionMgr.ui \
  src/code/display/QtPlotter/LegendPreferences.ui \
  src/code/display/QtPlotter/SmoothPreferences.ui \
  src/code/display/QtPlotter/QtProfileGUI.ui \
  src/code/display/QtPlotter/CanvasHolder.ui \
  src/code/display/QtPlotter/LineOverlaysTab.ui \
  src/code/display/QtPlotter/ColorSummaryWidget.ui \
  src/code/display/QtPlotter/QtProfilePrefsGUI.ui \
  src/code/display/QtPlotter/LineOverlaysSearchResultsDialog.ui \
  src/code/display/QtPlotter/SearchRedshiftDialog.ui \
  src/code/display/QtPlotter/SettingsWidget.ui \
  src/code/display/QtPlotter/SpectralPositioningWidget.ui \
  src/code/display/QtPlotter/annotations/AnnotationEditor.ui \
  src/code/display/QtPlotter/annotations/AnnotationEditorText.ui \
  src/code/display/QtViewer/QtExportManager.ui \
  src/code/display/QtViewer/ViewerPreferences.ui \
  src/code/display/QtViewer/ImageManager/ImageManagerDialog.ui \
  src/code/display/QtViewer/ImageManager/ImageView.ui \
  src/code/display/Fit/Fit2DTool.ui \
  src/code/display/Fit/FindSourcesDialog.ui \
  src/code/display/Fit/PixelRangeDialog.ui \
  src/code/display/Fit/ResidualHistogramDialog.ui \
  src/code/display/Fit/Fit2DLogDialog.ui \
  src/code/display/Slicer/SlicerMainWindow.ui \
  src/code/display/Slicer/ImageSlice.ui \
  src/code/guitools/Histogram/HistogramMain.ui \
  src/code/guitools/Histogram/ImageLoader.ui \
  src/code/guitools/Histogram/SaveHistogramWidget.ui \
  src/code/display/Clean/CleanGui.ui \
  src/code/display/QtViewer/QtExportBrowser.ui \
  src/code/display/region/HistogramTab.ui \
  src/code/guitools/Histogram/FitWidget.ui \
  src/code/guitools/Histogram/RangeControlsWidget.ui \
  src/code/guitools/Histogram/BinCountWidget.ui \
  src/code/guitools/Histogram/ChannelRangeWidget.ui \
  src/code/guitools/Histogram/ZoomWidget.ui \
  src/code/display/RegionShapes/QtSingleRegionShape.ui \
  src/code/display/RegionShapes/QtEditRegionShape.ui \
  src/code/display/RegionShapes/QtNewRegionShape.ui \
  src/code/display/QtPlotter/SearchMoleculesWidget.ui \
  src/code/display/QtPlotter/SearchMoleculesResultsWidget.ui \
  src/code/display/QtPlotter/SpecFitSettingsWidgetOptical.ui \
  src/code/display/QtPlotter/SpecFitSettingsWidgetRadio.ui \
  src/code/display/QtPlotter/GaussianEstimateDialog.ui \
  src/code/display/QtPlotter/SearchMoleculesDialog.ui \
  src/code/display/QtPlotter/MomentSettingsWidgetOptical.ui \
  src/code/display/QtPlotter/MomentSettingsWidgetRadio.ui \
  src/code/display/QtViewer/ImageManager/ImageScroll.ui \
  src/code/display/Slicer/SliceColorPreferences.ui \
  src/code/display/Slicer/SlicePlotPreferences.ui \
  src/code/display/Slicer/SliceSegment.ui \
  src/code/guitools/Histogram/ColorPreferences.ui \
  src/code/display/Clean/ConfirmStop.ui \
  src/code/display/region/HistogramGraph.ui \
  src/code/display/QtPlotter/SpecFitLogDialog.ui \
  src/code/display/QtPlotter/SpecFitSettingsFixedTableCell.ui \
  src/code/display/QtPlotter/GaussianEstimateWidget.ui \
  src/code/display/QtPlotter/ThresholdingBinPlotDialog.ui \
  src/code/display/RegionShapes/QtDS9WriteOptions.ui

TARGET = casaviewer

python_module.target = py
python_module.commands = scripts/generate-module prefix=build/python.$$QMAKE_HOST.version/casaviewer app=$$APP_EXE protoc=$$PROTOPY shutdown=$$PROTO_SHUTDOWN ping=$$PROTO_PING img=$$PROTO_IMG BUILD=$(BUILD)
python_module.depends = $$APP
QMAKE_EXTRA_TARGETS += python_module
